@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,7 +31,6 @@ import subprocess
 import time
 import cherrypy
 from threading import RLock, Lock, Condition, Thread
-from sabnzbd.lang import T, Ta
 
 #------------------------------------------------------------------------
 # Determine platform flags
@@ -63,25 +62,26 @@ elif os.name == 'posix':
 
 #------------------------------------------------------------------------
 
-import sabnzbd.nzbqueue as nzbqueue
-import sabnzbd.postproc as postproc
-import sabnzbd.downloader as downloader
-import sabnzbd.assembler as assembler
-import sabnzbd.newzbin as newzbin
+from sabnzbd.nzbqueue import NzbQueue
+from sabnzbd.postproc import PostProcessor
+from sabnzbd.downloader import Downloader
+from sabnzbd.assembler import Assembler
+from sabnzbd.newzbin import Bookmarks, MSGIDGrabber
 import sabnzbd.misc as misc
-import sabnzbd.dirscanner as dirscanner
-import sabnzbd.urlgrabber as urlgrabber
+from sabnzbd.dirscanner import DirScanner,  ProcessArchiveFile, ProcessSingleFile
+from sabnzbd.urlgrabber import URLGrabber
 import sabnzbd.scheduler as scheduler
 import sabnzbd.rss as rss
 import sabnzbd.emailer as emailer
-import sabnzbd.articlecache as articlecache
+from sabnzbd.articlecache import ArticleCache
 import sabnzbd.newsunpack
-import sabnzbd.codecs as codecs
+import sabnzbd.encoding as encoding
 import sabnzbd.config as config
-import sabnzbd.bpsmeter
+from sabnzbd.bpsmeter import BPSMeter
 import sabnzbd.cfg as cfg
 import sabnzbd.database
 import sabnzbd.lang as lang
+import sabnzbd.api
 from sabnzbd.decorators import *
 from sabnzbd.constants import *
 
@@ -98,13 +98,13 @@ DIR_LCLDATA = None
 DIR_PROG = None
 DIR_INTERFACES = None
 DIR_LANGUAGE = None
+DIR_PID = None
 
 QUEUECOMPLETE = None #stores the nice name of the action
 QUEUECOMPLETEACTION = None #stores the name of the function to be called
 QUEUECOMPLETEARG = None #stores an extra arguments that need to be passed
 QUEUECOMPLETEACTION_GO = False # Booleen value whether to run an action or not at the queue end.
 
-DEBUG_DELAY = 0
 DAEMON = None
 
 LOGFILE = None
@@ -113,6 +113,8 @@ LOGHANDLER = None
 GUIHANDLER = None
 LOG_ALL = False
 AMBI_LOCALHOST = False
+WIN_SERVICE = None      # Instance of our Win32 Service Class
+BROWSER_URL = None
 
 WEB_DIR = None
 WEB_DIR2 = None
@@ -124,11 +126,26 @@ RESTART_REQ = False
 OSX_ICON = 1
 PAUSED_ALL = False
 OLD_QUEUE = False
+SCHED_RESTART = False # Set when restarted through scheduler
 
 __INITIALIZED__ = False
 __SHUTTING_DOWN__ = False
 
 
+
+################################################################################
+# Table to map 0.5.x style language code to new style
+LANG_MAP = {
+    'de-de' : 'de',
+    'dk-da' : 'da', # Should have been "da-dk"
+    'fr-fr' : 'fr',
+    'nl-du' : 'nl', # Should have been "du-nl"
+    'no-no' : 'nb', # Norsk Bokmal
+    'sv-se' : 'sv',
+    'us-en' : 'en'  # Should have been "en-us"
+}
+
+
 ################################################################################
 # Signal Handler                                                               #
 ################################################################################
@@ -138,10 +155,15 @@ def sig_handler(signum = None, frame = None):
         # Ignore the "logoff" event when running as a Win32 daemon
         return True
     if type(signum) != type(None):
-        logging.warning(Ta('warn-signal@1'), signum)
+        logging.warning(Ta('Signal %s caught, saving and exiting...'), signum)
     try:
-        save_state()
+        save_state(flag=True)
     finally:
+        if sabnzbd.WIN32:
+            from util.apireg import del_connection_info
+            del_connection_info()
+        else:
+            pid_file()
         SABSTOP = True
         os._exit(0)
 
@@ -158,10 +180,9 @@ def connect_db(thread_index):
 
 
 @synchronized(INIT_LOCK)
-def initialize(pause_downloader = False, clean_up = False, evalSched=False):
+def initialize(pause_downloader = False, clean_up = False, evalSched=False, repair=0):
     global __INITIALIZED__, __SHUTTING_DOWN__,\
            LOGFILE, WEBLOGFILE, LOGHANDLER, GUIHANDLER, AMBI_LOCALHOST, WAITEXIT, \
-           DEBUG_DELAY, \
            DAEMON, MY_NAME, MY_FULLNAME, NEW_VERSION, \
            DIR_HOME, DIR_APPDATA, DIR_LCLDATA, DIR_PROG , DIR_INTERFACES, \
            DARWIN, RESTART_REQ, OSX_ICON, OLD_QUEUE
@@ -174,77 +195,93 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False):
     ### Set global database connection for Web-UI threads
     cherrypy.engine.subscribe('start_thread', connect_db)
 
-    ### Clean the cache folder, if requested
+    ### Clean-up, if requested
     if clean_up:
-        xlist= glob.glob(cfg.CACHE_DIR.get_path() + '/*')
-        for x in xlist:
-            os.remove(x)
+        # Old cache folder
+        misc.remove_all(cfg.cache_dir.get_path(), '*.sab')
+        misc.remove_all(cfg.cache_dir.get_path(), 'SABnzbd_*')
+        # New admin folder
+        misc.remove_all(cfg.admin_dir.get_path(), '*.sab')
 
     ### If dirscan_dir cannot be created, set a proper value anyway.
     ### Maybe it's a network path that's temporarily missing.
-    path = cfg.DIRSCAN_DIR.get_path()
+    path = cfg.dirscan_dir.get_path()
     if not os.path.exists(path):
-        sabnzbd.misc.create_real_path(cfg.DIRSCAN_DIR.ident(), '', path, False)
+        sabnzbd.misc.create_real_path(cfg.dirscan_dir.ident(), '', path, False)
 
     ### Set call backs for Config items
-    cfg.CACHE_LIMIT.callback(new_limit)
-    cfg.CHERRYHOST.callback(guard_restart)
-    cfg.CHERRYPORT.callback(guard_restart)
-    cfg.WEB_DIR.callback(guard_restart)
-    cfg.WEB_DIR2.callback(guard_restart)
-    cfg.WEB_COLOR.callback(guard_restart)
-    cfg.WEB_COLOR2.callback(guard_restart)
-    cfg.LOG_DIR.callback(guard_restart)
-    cfg.CACHE_DIR.callback(guard_restart)
-    cfg.HTTPS_PORT.callback(guard_restart)
-    cfg.HTTPS_CERT.callback(guard_restart)
-    cfg.HTTPS_KEY.callback(guard_restart)
-    cfg.ENABLE_HTTPS.callback(guard_restart)
-    cfg.BANDWIDTH_LIMIT.callback(guard_speedlimit)
-    cfg.TOP_ONLY.callback(guard_top_only)
-    cfg.PAUSE_ON_POST_PROCESSING.callback(guard_pause_on_pp)
+    cfg.cache_limit.callback(new_limit)
+    cfg.cherryhost.callback(guard_restart)
+    cfg.cherryport.callback(guard_restart)
+    cfg.web_dir.callback(guard_restart)
+    cfg.web_dir2.callback(guard_restart)
+    cfg.web_color.callback(guard_restart)
+    cfg.web_color2.callback(guard_restart)
+    cfg.log_dir.callback(guard_restart)
+    cfg.cache_dir.callback(guard_restart)
+    cfg.https_port.callback(guard_restart)
+    cfg.https_cert.callback(guard_restart)
+    cfg.https_key.callback(guard_restart)
+    cfg.enable_https.callback(guard_restart)
+    cfg.bandwidth_limit.callback(guard_speedlimit)
+    cfg.top_only.callback(guard_top_only)
+    cfg.pause_on_post_processing.callback(guard_pause_on_pp)
 
     ### Set cache limit
-    articlecache.method.new_limit(cfg.CACHE_LIMIT.get_int(), cfg.DEBUG_DELAY.get())
+    ArticleCache.do.new_limit(cfg.cache_limit.get_int())
+
+    ### Handle language upgrade from 0.5.x to 0.6.x
+    cfg.language.set(LANG_MAP.get(cfg.language(), cfg.language()))
 
     ### Set language files
-    lang.install_language(DIR_LANGUAGE, cfg.LANGUAGE.get())
+    lang.set_locale_info('SABnzbd', DIR_LANGUAGE)
+    lang.set_language(cfg.language())
+    sabnzbd.api.cache_skin_trans()
 
     ### Check for old queue (when a new queue is not present)
-    if not os.path.exists(os.path.join(cfg.CACHE_DIR.get_path(), QUEUE_FILE_NAME)):
-        OLD_QUEUE = bool(glob.glob(os.path.join(cfg.CACHE_DIR.get_path(), QUEUE_FILE_TMPL % '?')))
+    if not os.path.exists(os.path.join(cfg.cache_dir.get_path(), QUEUE_FILE_NAME)):
+        OLD_QUEUE = bool(misc.globber(cfg.cache_dir.get_path(), QUEUE_FILE_TMPL % '?'))
 
-    sabnzbd.change_queue_complete_action(cfg.queue_complete.get(), new=False)
+    sabnzbd.change_queue_complete_action(cfg.queue_complete(), new=False)
+
+    if check_repair_request():
+        repair = 2
+        pause_downloader = True
+    else:
+        # Check crash detection file
+        if load_admin(TERM_FLAG_FILE, remove=True):
+            # Repair mode 2 is a bit over an over-reaction!
+            pass # repair = 2
+
+    # Set crash detection file
+    save_admin(1, TERM_FLAG_FILE)
 
     ###
     ### Initialize threads
     ###
 
-    newzbin.bookmarks_init()
+    Bookmarks()
     rss.init()
+
+    BPSMeter.do.read()
+
+    PostProcessor()
+
+    NzbQueue()
+    NzbQueue.do.read_queue(repair)
+
+    Assembler()
+
+    Downloader(pause_downloader)
+
+    DirScanner()
+
+    MSGIDGrabber()
+
+    URLGrabber()
+
     scheduler.init()
 
-    bytes = load_data(BYTES_FILE_NAME, remove = False, do_pickle = False)
-    try:
-        bytes = int(bytes)
-        sabnzbd.bpsmeter.method.bytes_sum = bytes
-    except:
-        sabnzbd.bpsmeter.method.reset()
-
-    nzbqueue.init()
-
-    postproc.init()
-
-    assembler.init()
-
-    downloader.init(pause_downloader)
-
-    dirscanner.init()
-
-    newzbin.init_grabber()
-
-    urlgrabber.init()
-
     if evalSched:
         scheduler.analyse(pause_downloader)
 
@@ -260,23 +297,23 @@ def start():
 
     if __INITIALIZED__:
         logging.debug('Starting postprocessor')
-        postproc.start()
+        PostProcessor.do.start()
 
         logging.debug('Starting assembler')
-        assembler.start()
+        Assembler.do.start()
 
         logging.debug('Starting downloader')
-        downloader.start()
+        Downloader.do.start()
 
         scheduler.start()
 
         logging.debug('Starting dirscanner')
-        dirscanner.start()
+        DirScanner.do.start()
 
-        newzbin.start_grabber()
+        MSGIDGrabber.do.start()
 
         logging.debug('Starting urlgrabber')
-        urlgrabber.start()
+        URLGrabber.do.start()
 
 
 @synchronized(INIT_LOCK)
@@ -289,29 +326,50 @@ def halt():
 
         rss.stop()
 
-        newzbin.bookmarks_save()
+        Bookmarks.do.save()
 
         logging.debug('Stopping URLGrabber')
-        urlgrabber.stop()
+        URLGrabber.do.stop()
+        try:
+            URLGrabber.do.join()
+        except:
+            pass
 
         logging.debug('Stopping Newzbin-Grabber')
-        newzbin.stop_grabber()
+        MSGIDGrabber.do.stop()
+        try:
+            MSGIDGrabber.do.join()
+        except:
+            pass
 
         logging.debug('Stopping dirscanner')
-        dirscanner.stop()
+        DirScanner.do.stop()
+        try:
+            DirScanner.do.join()
+        except:
+            pass
+
 
         ## Stop Required Objects ##
         logging.debug('Stopping downloader')
-        downloader.stop()
+        sabnzbd.downloader.stop()
 
         logging.debug('Stopping assembler')
-        assembler.stop()
+        Assembler.do.stop()
+        try:
+            Assembler.do.join()
+        except:
+            pass
 
         logging.debug('Stopping postprocessor')
-        postproc.stop()
+        PostProcessor.do.stop()
+        try:
+            PostProcessor.do.join()
+        except:
+            pass
 
         ## Save State ##
-        save_state()
+        save_state(flag=True)
 
         # The Scheduler cannot be stopped when the stop was scheduled.
         # Since all warm-restarts have been removed, it's not longer
@@ -331,7 +389,7 @@ def halt():
 
 def new_limit():
     """ Callback for article cache changes """
-    articlecache.method.new_limit(cfg.CACHE_LIMIT.get_int())
+    ArticleCache.do.new_limit(cfg.cache_limit.get_int())
 
 def guard_restart():
     """ Callback for config options requiring a restart """
@@ -340,68 +398,79 @@ def guard_restart():
 
 def guard_speedlimit():
     """ Callback for change of bandwidth_limit, sets actual speed """
-    downloader.limit_speed(cfg.BANDWIDTH_LIMIT.get())
+    Downloader.do.limit_speed(cfg.bandwidth_limit())
 
 def guard_top_only():
     """ Callback for change of top_only option """
-    nzbqueue.set_top_only(cfg.TOP_ONLY.get())
+    NzbQueue.do.set_top_only(cfg.top_only())
 
 def guard_pause_on_pp():
     """ Callback for change of pause-download-on-pp """
-    if cfg.PAUSE_ON_POST_PROCESSING.get():
+    if cfg.pause_on_post_processing():
         pass # Not safe to idle downloader, because we don't know
              # if post-processing is active now
     else:
-        downloader.unidle_downloader()
+        Downloader.do.resume_from_postproc()
 
 def add_msgid(msgid, pp=None, script=None, cat=None, priority=None, nzbname=None):
-
+    """ Add NZB based on newzbin report number, attributes optional
+    """
     if pp and pp=="-1": pp = None
     if script and script.lower()=='default': script = None
     if cat and cat.lower()=='default': cat = None
 
-    if cfg.USERNAME_NEWZBIN.get() and cfg.PASSWORD_NEWZBIN.get():
+    if cfg.newzbin_username() and cfg.newzbin_password():
         logging.info('Fetching msgid %s from www.newzbin.com', msgid)
-        msg = T('fetchingNewzbin@1') % msgid
+        msg = T('fetching msgid %s from www.newzbin.com') % msgid
 
-        future_nzo = nzbqueue.generate_future(msg, pp, script, cat=cat, url=msgid, priority=priority, nzbname=nzbname)
+        future_nzo = NzbQueue.do.generate_future(msg, pp, script, cat=cat, url=msgid, priority=priority, nzbname=nzbname)
 
-        newzbin.grab(msgid, future_nzo)
+        MSGIDGrabber.do.grab(msgid, future_nzo)
     else:
-        logging.error(Ta('error-fetchNewzbin@1'), msgid)
+        logging.error(Ta('Error Fetching msgid %s from www.newzbin.com - Please make sure your Username and Password are set'), msgid)
 
 
 def add_url(url, pp=None, script=None, cat=None, priority=None, nzbname=None):
+    """ Add NZB based on a URL, attributes optional
+    """
     if 'http' not in url:
         return
     if pp and pp=="-1": pp = None
     if script and script.lower()=='default': script = None
     if cat and cat.lower()=='default': cat = None
     logging.info('Fetching %s', url)
-    msg = T('fetchNZB@1') % url
-    future_nzo = nzbqueue.generate_future(msg, pp, script, cat, url=url, priority=priority, nzbname=nzbname)
-    urlgrabber.add(url, future_nzo)
+    msg = T('Trying to fetch NZB from %s') % url
+    future_nzo = NzbQueue.do.generate_future(msg, pp, script, cat, url=url, priority=priority, nzbname=nzbname)
+    URLGrabber.do.add(url, future_nzo)
 
 
-def save_state():
-    articlecache.method.flush_articles()
-    nzbqueue.save()
-    save_data(str(sabnzbd.bpsmeter.method.get_sum()), BYTES_FILE_NAME, do_pickle = False)
+def save_state(flag=False):
+    """ Save all internal bookkeeping to disk """
+    ArticleCache.do.flush_articles()
+    NzbQueue.do.save()
+    BPSMeter.do.save()
     rss.save()
-    newzbin.bookmarks_save()
-    dirscanner.save()
-    postproc.save()
+    Bookmarks.do.save()
+    DirScanner.do.save()
+    PostProcessor.do.save()
+    if flag:
+        # Remove crash detector
+        load_admin(TERM_FLAG_FILE, remove=True)
 
 def pause_all():
+    """ Pause all activities than cause disk access
+    """
     global PAUSED_ALL
     PAUSED_ALL = True
-    sabnzbd.downloader.pause_downloader()
+    Downloader.do.pause()
     logging.debug('PAUSED_ALL active')
 
 def unpause_all():
+    """ Resume all activcities
+    """
     global PAUSED_ALL
     PAUSED_ALL = False
-    sabnzbd.downloader.resume_downloader()
+    Downloader.do.resume()
     logging.debug('PAUSED_ALL inactive')
 
 
@@ -414,47 +483,66 @@ NZB_LOCK = Lock()
 def backup_exists(filename):
     """ Return True if backup exists and no_dupes is set
     """
-    path = cfg.NZB_BACKUP_DIR.get_path()
-    return path and sabnzbd.cfg.NO_DUPES.get() and \
+    path = cfg.nzb_backup_dir.get_path()
+    return path and sabnzbd.cfg.no_dupes() and \
            os.path.exists(os.path.join(path, filename+'.gz'))
 
-@synchronized(NZB_LOCK)
 def backup_nzb(filename, data):
     """ Backup NZB file
     """
-    if cfg.NZB_BACKUP_DIR.get_path():
-        backup_name = filename + '.gz'
+    path = cfg.nzb_backup_dir.get_path()
+    if path:
+        save_compressed(path, filename, data)
 
-        # Need to go to the backup folder to
-        # prevent the pathname being embedded in the GZ file
-        here = os.getcwd()
-        os.chdir(cfg.NZB_BACKUP_DIR.get_path())
 
-        logging.info("Backing up %s", backup_name)
-        try:
-            _f = gzip.GzipFile(backup_name, 'wb')
-            _f.write(data)
-            _f.flush()
-            _f.close()
-        except:
-            logging.error("Saving %s to %s failed", backup_name, cfg.NZB_BACKUP_DIR.get_path())
-            logging.info("Traceback: ", exc_info = True)
+@synchronized(NZB_LOCK)
+def save_compressed(folder, filename, data):
+    """ Save compressed NZB file in folder
+    """
+    # Need to go to the save folder to
+    # prevent the pathname being embedded in the GZ file
+    here = os.getcwd()
+    os.chdir(folder)
 
-        os.chdir(here)
+    if filename.endswith('.nzb'):
+        filename += '.gz'
+    else:
+        filename += '.nzb.gz'
+    logging.info("Backing up %s", os.path.join(folder, filename))
+    try:
+        f = gzip.GzipFile(filename, 'wb')
+        f.write(data)
+        f.flush()
+        f.close()
+    except:
+        logging.error("Saving %s failed", os.path.join(folder, filename))
+        logging.info("Traceback: ", exc_info = True)
+
+    os.chdir(here)
 
 
 ################################################################################
 ## CV synchronized (notifies downloader)                                      ##
 ################################################################################
 @synchronized_CV
-def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None):
+def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None, reuse=False):
+    """ Add disk-based NZB file, optional attributes,
+        'reuse' flag will suppress duplicate detection
+    """
     if pp and pp=="-1": pp = None
     if script and script.lower()=='default': script = None
     if cat and cat.lower()=='default': cat = None
 
-    # Consider reception of Latin-1 names for non-Windows platforms
-    # When an OSX/Unix server receives a file from Windows platform
-    filename = codecs.special_fixer(nzbfile.filename)
+    if isinstance(nzbfile, str):
+        # File coming from queue repair
+        filename = nzbfile
+        keep = True
+    else:
+        # File coming from API/TAPI
+        # Consider reception of Latin-1 names for non-Windows platforms
+        # When an OSX/Unix server receives a file from Windows platform
+        filename = encoding.special_fixer(nzbfile.filename)
+        keep = False
 
     if not sabnzbd.WIN32:
         # If windows client sends file to Unix server backslashed may
@@ -466,45 +554,53 @@ def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORIT
 
     logging.info('Adding %s', filename)
 
-    try:
-        f, path = tempfile.mkstemp(suffix=ext, text=False)
-        os.write(f, nzbfile.value)
-        os.close(f)
-    except:
-        logging.error(Ta('error-tempFile@1'), filename)
-        logging.info("Traceback: ", exc_info = True)
+    if isinstance(nzbfile, str):
+        path = nzbfile
+    else:
+        try:
+            f, path = tempfile.mkstemp(suffix=ext, text=False)
+            os.write(f, nzbfile.value)
+            os.close(f)
+        except:
+            logging.error(Ta('Cannot create temp file for %s'), filename)
+            logging.info("Traceback: ", exc_info = True)
 
     if ext.lower() in ('.zip', '.rar'):
-        dirscanner.ProcessArchiveFile(filename, path, pp, script, cat, priority=priority)
+        ProcessArchiveFile(filename, path, pp, script, cat, priority=priority)
     else:
-        dirscanner.ProcessSingleFile(filename, path, pp, script, cat, priority=priority, nzbname=nzbname)
+        ProcessSingleFile(filename, path, pp, script, cat, priority=priority, nzbname=nzbname, keep=keep, reuse=reuse)
 
 
 ################################################################################
 ## Unsynchronized methods                                                     ##
 ################################################################################
 def enable_server(server):
+    """ Enable server (scheduler only)
+    """
     try:
         config.get_config('servers', server).enable.set(1)
     except:
-        logging.warning(Ta('warn-noServer@1'), server)
+        logging.warning(Ta('Trying to set status of non-existing server %s'), server)
         return
     config.save_config()
-    downloader.update_server(server, server)
+    Downloader.do.update_server(server, server)
 
 
 def disable_server(server):
-    """ Disable server """
+    """ Disable server (scheduler only)
+    """
     try:
         config.get_config('servers', server).enable.set(0)
     except:
-        logging.warning(Ta('warn-noServer@1'), server)
+        logging.warning(Ta('Trying to set status of non-existing server %s'), server)
         return
     config.save_config()
-    downloader.update_server(server, server)
+    Downloader.do.update_server(server, server)
 
 
 def system_shutdown():
+    """ Shutdown system after halting download and saving bookkeeping
+    """
     logging.info("Performing system shutdown")
 
     Thread(target=halt).start()
@@ -520,6 +616,7 @@ def system_shutdown():
 
 
 def system_hibernate():
+    """ Hibernate system """
     logging.info("Performing system hybernation")
     if sabnzbd.WIN32:
         misc.win_hibernate()
@@ -530,6 +627,7 @@ def system_hibernate():
 
 
 def system_standby():
+    """ Standby system """
     logging.info("Performing system standby")
     if sabnzbd.WIN32:
         misc.win_standby()
@@ -540,6 +638,7 @@ def system_standby():
 
 
 def shutdown_program():
+    """ Stop program after halting and saving """
     logging.info("Performing sabnzbd shutdown")
     Thread(target=halt).start()
     while __INITIALIZED__:
@@ -549,11 +648,11 @@ def shutdown_program():
 
 def restart_program():
     """ Restart program (used by scheduler) """
-    logging.info("Performing sabnzbd restart")
-    sabnzbd.halt()
-    while __INITIALIZED__:
-        time.sleep(1.0)
-    cherrypy.engine.restart()
+    global SCHED_RESTART
+    logging.info("Scheduled restart request")
+    # Just set the stop flag, because stopping CherryPy from
+    # the scheduler is not reliable
+    cherrypy.engine.execv = SCHED_RESTART = True
 
 
 def change_queue_complete_action(action, new=True):
@@ -584,6 +683,10 @@ def change_queue_complete_action(action, new=True):
     else:
         action = None
 
+    if new:
+        cfg.queue_complete.set(action or '')
+        config.save_config()
+
     #keep the name of the action for matching the current select in queue.tmpl
     QUEUECOMPLETE = action
 
@@ -592,7 +695,8 @@ def change_queue_complete_action(action, new=True):
 
 
 def run_script(script):
-    command = os.path.join(cfg.SCRIPT_DIR.get_path(), script)
+    """ Run a user script (queue complete only) """
+    command = [os.path.join(cfg.script_dir.get_path(), script)]
     stup, need_shell, command, creationflags = sabnzbd.newsunpack.build_command(command)
     logging.info('Spawning external command %s', command)
     subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
@@ -603,25 +707,27 @@ def run_script(script):
 def empty_queues():
     """ Return True if queues empty or non-existent """
     global __INITIALIZED__
-    return (not __INITIALIZED__) or (postproc.empty() and not nzbqueue.has_articles())
+    return (not __INITIALIZED__) or (PostProcessor.do.empty() and NzbQueue.do.is_empty())
 
 
 def keep_awake():
     """ If we still have work to do, keep Windows system awake
     """
     global KERNEL32
-    if KERNEL32 and not downloader.paused():
-        if (not postproc.empty()) or nzbqueue.has_articles():
+    if KERNEL32 and not sabnzbd.downloader.Downloader.do.paused:
+        if (not PostProcessor.do.empty()) or not NzbQueue.do.is_empty():
             # set ES_SYSTEM_REQUIRED
             KERNEL32.SetThreadExecutionState(ctypes.c_int(0x00000001))
 
 
 def CheckFreeSpace():
-    if cfg.DOWNLOAD_FREE.get() and not downloader.paused():
-        if misc.diskfree(cfg.DOWNLOAD_DIR.get_path()) < cfg.DOWNLOAD_FREE.get_float() / GIGI:
-            logging.warning(Ta('warn-noSpace'))
+    """ Check if enough disk space is free, if not pause downloader and send email
+    """
+    if cfg.download_free() and not sabnzbd.downloader.Downloader.do.paused:
+        if misc.diskfree(cfg.download_dir.get_path()) < cfg.download_free.get_float() / GIGI:
+            logging.warning(Ta('Too little diskspace forcing PAUSE'))
             # Pause downloader, but don't save, since the disk is almost full!
-            downloader.pause_downloader(save=False)
+            Downloader.do.pause(save=False)
             emailer.diskfull()
 
 
@@ -631,22 +737,32 @@ def CheckFreeSpace():
 IO_LOCK = RLock()
 
 @synchronized(IO_LOCK)
-def get_new_id(prefix):
-    try:
-        fd, l = tempfile.mkstemp('', 'SABnzbd_%s_' % prefix, cfg.CACHE_DIR.get_path())
-        os.close(fd)
-        head, tail = os.path.split(l)
-        return tail
-    except:
-        logging.error(Ta('error-failMkstemp'))
-        logging.info("Traceback: ", exc_info = True)
+def get_new_id(prefix, folder, check_list=None):
+    """ Return unique prefixed admin identifier within folder
+        optionally making sure that id is not in the check_list.
+    """
+    for n in xrange(10000):
+        try:
+            if not os.path.exists(folder):
+                os.makedirs(folder)
+            fd, path = tempfile.mkstemp('', 'SABnzbd_%s_' % prefix, folder)
+            os.close(fd)
+            head, tail = os.path.split(path)
+            if not check_list or tail not in check_list:
+                return tail
+        except:
+            logging.error(Ta('Failure in tempfile.mkstemp'))
+            logging.info("Traceback: ", exc_info = True)
+    # Cannot create unique id, crash the process
+    raise IOError
 
 
 @synchronized(IO_LOCK)
-def save_data(data, _id, do_pickle = True, doze=0, silent=False):
-    path = os.path.join(cfg.CACHE_DIR.get_path(), _id)
+def save_data(data, _id, path, do_pickle = True, silent=False):
+    """ Save data to a diskfile """
     if not silent:
         logging.debug("Saving data for %s in %s", _id, path)
+    path = os.path.join(path, _id)
 
     try:
         _f = open(path, 'wb')
@@ -654,27 +770,24 @@ def save_data(data, _id, do_pickle = True, doze=0, silent=False):
             cPickle.dump(data, _f, 2)
         else:
             _f.write(data)
-        if doze:
-            # Only for debugging decoder overflow
-            time.sleep(doze)
         _f.flush()
         _f.close()
     except:
-        logging.error(Ta('error-saveX@1'), path)
+        logging.error(Ta('Saving %s failed'), path)
         logging.info("Traceback: ", exc_info = True)
 
 
 @synchronized(IO_LOCK)
-def load_data(_id, remove = True, do_pickle = True, silent=False):
-    path = os.path.join(cfg.CACHE_DIR.get_path(), _id)
-    if not silent:
-        logging.debug("Loading data for %s from %s", _id, path)
+def load_data(_id, path, remove=True, do_pickle=True, silent=False):
+    """ Read data from disk file """
+    path = os.path.join(path, _id)
 
     if not os.path.exists(path):
         logging.info("%s missing", path)
         return None
 
-    data = None
+    if not silent:
+        logging.debug("Loading data for %s from %s", _id, path)
 
     try:
         _f = open(path, 'rb')
@@ -685,9 +798,9 @@ def load_data(_id, remove = True, do_pickle = True, silent=False):
         _f.close()
 
         if remove:
-            remove_data(_id)
+            os.remove(path)
     except:
-        logging.error(Ta('error-loading@1'), path)
+        logging.error(Ta('Loading %s failed'), path)
         logging.info("Traceback: ", exc_info = True)
         return None
 
@@ -695,19 +808,75 @@ def load_data(_id, remove = True, do_pickle = True, silent=False):
 
 
 @synchronized(IO_LOCK)
-def remove_data(_id):
-    path = os.path.join(cfg.CACHE_DIR.get_path(), _id)
+def remove_data(_id, path):
+    """ Remove admin file """
+    path = os.path.join(path, _id)
     try:
-        os.remove(path)
-        logging.info("%s removed", path)
+        if os.path.exists(path):
+            os.remove(path)
+            logging.info("%s removed", path)
     except:
-        pass
+        logging.info("Failed to remove %s", path)
+        logging.info("Traceback: ", exc_info = True)
+
+
+
+@synchronized(IO_LOCK)
+def save_admin(data, _id, do_pickle=True):
+    """ Save data in admin folder in specified format """
+    path = os.path.join(cfg.admin_dir.get_path(), _id)
+    logging.info("Saving data for %s in %s", _id, path)
+
+    try:
+        f = open(path, 'wb')
+        if do_pickle:
+            cPickle.dump(data, f, 2)
+        else:
+            f.write(data)
+        f.flush()
+        f.close()
+    except:
+        logging.error(Ta('Saving %s failed'), path)
+        logging.info("Traceback: ", exc_info = True)
+
+
+@synchronized(IO_LOCK)
+def load_admin(_id, remove=False, do_pickle=True):
+    """ Read data in admin folder in specified format """
+    path = os.path.join(cfg.admin_dir.get_path(), _id)
+    logging.info("Loading data for %s from %s", _id, path)
+
+    if not os.path.exists(path):
+        logging.info("%s missing, trying old cache", path)
+        path = os.path.join(cfg.cache_dir.get_path(), _id)
+        if not os.path.exists(path):
+            logging.info("%s missing", path)
+            return None
+        remove = True
+
+    try:
+        f = open(path, 'rb')
+        if do_pickle:
+            data = cPickle.load(f)
+        else:
+            data = f.read()
+        f.close()
+
+        if remove:
+            os.remove(path)
+    except:
+        logging.error(Ta('Loading %s failed'), path)
+        logging.info("Traceback: ", exc_info = True)
+        return None
+
+    return data
+
 
 
 def pp_to_opts(pp):
     """ Convert numeric processinf options to (repair, unpack, delete) """
     # Convert the pp to an int
-    pp = sabnzbd.interface.IntConv(pp)
+    pp = sabnzbd.interface.int_conv(pp)
     if pp == 0 : return (False, False, False)
     if pp == 1 : return (True, False, False)
     if pp == 2 : return (True, True, False)
@@ -719,12 +888,34 @@ def opts_to_pp(repair, unpack, delete):
     if repair is None:
         return None
     pp = 0
-    if repair: pp += 1
-    if unpack: pp += 1
-    if delete: pp += 1
+    if repair: pp = 1
+    if unpack: pp = 2
+    if delete: pp = 3
     return pp
 
 
+def request_repair():
+    """ Request a full repair on next restart """
+    path = os.path.join(cfg.admin_dir.get_path(), REPAIR_REQUEST)
+    try:
+        f = open(path, 'w')
+        f.write('\n')
+        f.close()
+    except:
+        pass
+
+def check_repair_request():
+    """ Return True if repair request found, remove afterwards """
+    path = os.path.join(cfg.admin_dir.get_path(), REPAIR_REQUEST)
+    if os.path.exists(path):
+        try:
+            os.remove(path)
+        except:
+            pass
+        return True
+    return False
+
+
 def SimpleRarExtract(rarfile, fn):
     """ Wrapper for call to newsunpack, required to avoid circular imports
     """
@@ -739,36 +930,36 @@ def check_all_tasks():
         return True
 
     # Non-restartable threads, require program restart
-    if not sabnzbd.postproc.alive():
+    if not sabnzbd.PostProcessor.do.isAlive():
         logging.info('Restarting because of crashed postprocessor')
         return False
-    if not sabnzbd.downloader.alive():
+    if not Downloader.do.isAlive():
         logging.info('Restarting because of crashed downloader')
         return False
-    if not sabnzbd.assembler.alive():
+    if not Assembler.do.isAlive():
         logging.info('Restarting because of crashed assembler')
         return False
 
     # Kick the downloader, in case it missed the semaphore
-    sabnzbd.downloader.wakeup()
+    Downloader.do.wakeup()
 
     # Make sure the right servers are active
-    sabnzbd.downloader.check()
+    Downloader.do.check_timers()
 
     # Restartable threads
-    if not sabnzbd.dirscanner.alive():
+    if not DirScanner.do.isAlive():
         logging.info('Restarting crashed dirscanner')
-        sabnzbd.dirscanner.init()
-    if not sabnzbd.urlgrabber.alive():
+        DirScanner.do.__init__()
+    if not URLGrabber.do.isAlive():
         logging.info('Restarting crashed urlgrabber')
-        sabnzbd.urlgrabber.init()
-    if not sabnzbd.newzbin.alive():
+        URLGrabber.do.__init__()
+    if not MSGIDGrabber.do.isAlive():
         logging.info('Restarting crashed newzbin')
-        sabnzbd.newzbin.init_grabber()
+        MSGIDGrabber.do.__init__()
     if not sabnzbd.scheduler.sched_check():
         logging.info('Restarting crashed scheduler')
         sabnzbd.scheduler.init()
-        sabnzbd.downloader.unblock_all()
+        sabnzbd.downloader.Downloader.do.unblock_all()
 
     # Check one-shot pause
     sabnzbd.scheduler.pause_check()
@@ -776,6 +967,45 @@ def check_all_tasks():
     return True
 
 
+def pid_file(pid_path=None, port=0):
+    """ Create or remove pid file
+    """
+    global DIR_PID
+    if not sabnzbd.WIN32 and pid_path and pid_path.startswith('/'):
+        DIR_PID = os.path.join(pid_path, 'sabnzbd-%s.pid' % port)
+
+    if DIR_PID:
+        try:
+            if port:
+                f = open(DIR_PID, 'w')
+                f.write('%d\n' % os.getpid())
+                f.close()
+            else:
+                os.remove(DIR_PID)
+        except:
+            logging.warning('Cannot access PID file %s', DIR_PID)
+
+
+
 # Required wrapper because nzbstuff.py cannot import downloader.py
 def active_primaries():
-    return sabnzbd.downloader.active_primaries()
+    return sabnzbd.downloader.Downloader.do.active_primaries()
+
+
+def proxy_postproc(nzo):
+    sabnzbd.postproc.PostProcessor.do.process(nzo)
+
+def proxy_pre_queue(name, pp, cat, script, priority, size, groups):
+    return sabnzbd.newsunpack.pre_queue(name, pp, cat, script, priority, size, groups)
+
+def proxy_get_history_size():
+    history_db = sabnzbd.database.get_history_handle()
+    return history_db.get_history_size()
+
+def proxy_build_history():
+    """ Proxy to let nzbqueue call api """
+    return sabnzbd.api.build_history()
+
+def proxy_rm_bookmark(url):
+    """ Proxy to urlgrabber rm_bookmark """
+    return sabnzbd.urlgrabber.URLGrabber.do.rm_bookmark(url)
