@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-CONFIG_VERSION = 18
+CONFIG_VERSION = 19
+
 QUEUE_VERSION = 9
 POSTPROC_QUEUE_VERSION = 1
 
@@ -37,6 +38,7 @@ PNFO_ACTIVE_FILES_FIELD = 14
 PNFO_QUEUED_FILES_FIELD = 15
 PNFO_STATUS_FIELD = 16
 PNFO_PRIORITY_FIELD = 17
+PNFO_MISSING_FIELD = 18
 
 QNFO_BYTES_FIELD = 0
 QNFO_BYTES_LEFT_FIELD = 1
@@ -50,13 +52,18 @@ GIGI = float(2 ** 30)
 MEBI = float(2 ** 20)
 KIBI = float(2 ** 10)
 
-BYTES_FILE_NAME  = 'bytes%s.sab' % QUEUE_VERSION
+BYTES_FILE_NAME  = 'totals%s.sab' % QUEUE_VERSION
 QUEUE_FILE_TMPL  = 'queue%s.sab'
 QUEUE_FILE_NAME  =  QUEUE_FILE_TMPL % QUEUE_VERSION
 POSTPROC_QUEUE_FILE_NAME  = 'postproc%s.sab' % POSTPROC_QUEUE_VERSION
 RSS_FILE_NAME    = 'rss_data.sab'
 BOOKMARK_FILE_NAME = 'bookmarks.sab'
 SCAN_FILE_NAME    = 'watched_data.sab'
+TERM_FLAG_FILE    = 'running.sab'
+FUTURE_Q_FOLDER   = 'future'
+JOB_ADMIN = '__ADMIN__'
+ATTRIB_FILE = 'SABnzbd_attrib'
+REPAIR_REQUEST = 'repair-all.sab'
 
 DB_HISTORY_VERSION = 1
 DB_QUEUE_VERSION = 1
@@ -64,16 +71,18 @@ DB_QUEUE_VERSION = 1
 DB_HISTORY_NAME = 'history%s.db' % DB_HISTORY_VERSION
 DB_QUEUE_NAME = 'queue%s.db' % DB_QUEUE_VERSION
 
-DEF_DOWNLOAD_DIR = 'downloads/incomplete'
-DEF_COMPLETE_DIR = 'downloads/complete'
+DEF_DOWNLOAD_DIR = 'Downloads/incomplete'
+DEF_COMPLETE_DIR = 'Downloads/complete'
 DEF_CACHE_DIR    = 'cache'
+DEF_ADMIN_DIR    = 'admin'
 DEF_LOG_DIR      = 'logs'
 DEF_NZBBACK_DIR  = ''
-DEF_LANGUAGE     = 'language'
+DEF_LANGUAGE     = 'locale'
 DEF_INTERFACES   = 'interfaces'
-DEF_INT_LANGUAGE = 'language'
-DEF_STDINTF      = 'smpl'
-DEF_SKIN_COLORS  = {'smpl':'white', 'classic':'darkblue', 'mobile':'light'}
+DEF_INT_LANGUAGE = 'locale'
+DEF_EMAIL_TMPL   = 'email'
+DEF_STDINTF      = 'Plush'
+DEF_SKIN_COLORS  = {'smpl':'white', 'classic':'darkblue', 'mobile':'light', 'plush' : 'gold'}
 DEF_MAIN_TMPL    = 'templates/main.tmpl'
 DEF_INI_FILE     = 'sabnzbd.ini'
 DEF_HOST         = 'localhost'
@@ -100,6 +109,8 @@ HIGH_PRIORITY = 1
 NORMAL_PRIORITY = 0
 LOW_PRIORITY = -1
 DEFAULT_PRIORITY = -100
+PAUSED_PRIORITY = -2
+DUP_PRIORITY = -3
 
 #(MATCHER, [EXTRA,MATCHERS])
 series_match = [ (r'( [sS]|[\d]+)x(\d+)', # 1x01
