@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,15 +30,14 @@ except:
 import os
 import time
 import datetime
-from calendar import MONDAY
 import zlib
 import logging
 
 import sabnzbd
 import sabnzbd.cfg
 from sabnzbd.constants import DB_HISTORY_NAME
-from sabnzbd.lang import T, Ta
-from sabnzbd.codecs import unicoder
+from sabnzbd.encoding import unicoder
+from sabnzbd.bpsmeter import this_week, this_month
 
 _HISTORY_DB = None        # Will contain full path to history database
 _DONE_CLEANING = False    # Ensure we only do one Vacuum per session
@@ -47,13 +46,13 @@ def get_history_handle():
     """ Get an instance of the history db hanlder """
     global _HISTORY_DB
     if not _HISTORY_DB:
-        _HISTORY_DB = os.path.join(sabnzbd.cfg.ADMIN_DIR.get_path(), DB_HISTORY_NAME)
+        _HISTORY_DB = os.path.join(sabnzbd.cfg.admin_dir.get_path(), DB_HISTORY_NAME)
     return HistoryDB(_HISTORY_DB)
 
 
 # Note: Add support for execute return values
 
-class HistoryDB:
+class HistoryDB(object):
     def __init__(self, db_path):
         global _DONE_CLEANING
         #Thread.__init__(self)
@@ -87,13 +86,13 @@ class HistoryDB:
                 self.save()
             return True
         except:
-            logging.error(Ta('error-sqlCmd'))
-            logging.info("SQL: %s" , command)
+            logging.error(Ta('SQL Command Failed, see log'))
+            logging.debug("SQL: %s" , command)
             logging.info("Traceback: ", exc_info = True)
             try:
                 self.con.rollback()
             except:
-                logging.info("Rollback Failed:", exc_info = True)
+                logging.debug("Rollback Failed:", exc_info = True)
             return False
 
     def create_history_db(self):
@@ -130,26 +129,26 @@ class HistoryDB:
         try:
             self.con.commit()
         except:
-            logging.error(Ta('error-sqlCommit'))
-            logging.debug("Traceback: ", exc_info = True)
+            logging.error(Ta('SQL Commit Failed, see log'))
+            logging.info("Traceback: ", exc_info = True)
 
     def close(self):
         try:
             self.c.close()
             self.con.close()
         except:
-            logging.error(Ta('error-sqlClose'))
-            logging.debug("Traceback: ", exc_info = True)
+            logging.error(Ta('Failed to close database, see log'))
+            logging.info("Traceback: ", exc_info = True)
 
-    def remove_all(self):
-        return self.execute("""DELETE FROM history""")
+    def remove_completed(self):
+        return self.execute("""DELETE FROM history WHERE status = 'Completed'""", save=True)
 
     def remove_failed(self):
-        return self.execute('''DELETE FROM history WHERE status="Failed"''', save=True)
+        return self.execute("""DELETE FROM history WHERE status = 'Failed'""", save=True)
 
     def remove_history(self, jobs=None):
         if jobs is None:
-            self.remove_all()
+            self.remove_completed()
         else:
             if type(jobs) == type(''):
                 jobs = [jobs]
@@ -170,7 +169,7 @@ class HistoryDB:
         VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", t):
             self.save()
 
-    def fetch_history(self, start=None, limit=None, search=None):
+    def fetch_history(self, start=None, limit=None, search=None, failed_only=0):
 
         if not search:
             # Default value
@@ -190,8 +189,12 @@ class HistoryDB:
             search = '%' + search + '%'
 
         # Get the number of results
-        if self.execute('select count(*) from History WHERE name LIKE ?', (search,)):
-            total_items = self.c.fetchone()['count(*)']
+        if failed_only:
+            res = self.execute('select count(*) from History WHERE name LIKE ? AND STATUS = "Failed"', (search,))
+        else:
+            res = self.execute('select count(*) from History WHERE name LIKE ?', (search,))
+        if res:
+            total_items = self.c.fetchone().get('count(*)')
         else:
             total_items = -1
 
@@ -200,8 +203,11 @@ class HistoryDB:
         if not limit:
             limit = total_items
 
-        t = (search, start,limit)
-        fetch_ok = self.execute("""SELECT * FROM history WHERE name LIKE ? ORDER BY completed desc LIMIT ?, ?""", t)
+        t = (search, start, limit)
+        if failed_only:
+            fetch_ok = self.execute('''SELECT * FROM history WHERE name LIKE ? AND STATUS = "Failed" ORDER BY completed desc LIMIT ?, ?''', t)
+        else:
+            fetch_ok = self.execute('''SELECT * FROM history WHERE name LIKE ? ORDER BY completed desc LIMIT ?, ?''', t)
 
         if fetch_ok:
             items = self.c.fetchall()
@@ -224,27 +230,27 @@ class HistoryDB:
         # Total Size of the history
         if self.execute('''SELECT sum(bytes) FROM history'''):
             f = self.c.fetchone()
-            total = f['sum(bytes)']
+            total = f.get('sum(bytes)')
         else:
             total = 0
 
         # Amount downloaded this month
-        r = time.gmtime(time.time())
-        month_timest = int(time.mktime((r.tm_year, r.tm_mon, 0, 0, 0, 1, r.tm_wday, r.tm_yday, r.tm_isdst)))
+        #r = time.gmtime(time.time())
+        #month_timest = int(time.mktime((r.tm_year, r.tm_mon, 0, 0, 0, 1, r.tm_wday, r.tm_yday, r.tm_isdst)))
+        month_timest = int(this_month(time.time()))
 
         if self.execute('''SELECT sum(bytes) FROM history WHERE "completed">?''', (month_timest,)):
             f = self.c.fetchone()
-            month = f['sum(bytes)']
+            month = f.get('sum(bytes)')
         else:
             month = 0
 
         # Amount downloaded this week
-        monday = find_monday()
-        week_timest = int(time.mktime(find_monday()))
+        week_timest = int(this_week(time.time()))
 
         if self.execute('''SELECT sum(bytes) FROM history WHERE "completed">?''', (week_timest,)):
             f = self.c.fetchone()
-            week = f['sum(bytes)']
+            week = f.get('sum(bytes)')
         else:
             week = 0
 
@@ -252,36 +258,48 @@ class HistoryDB:
 
 
     def get_script_log(self, nzo_id):
+        data = ''
         t = (nzo_id,)
         if self.execute('SELECT script_log FROM history WHERE nzo_id=?', t):
             f = self.c.fetchone()
-            return zlib.decompress(f['script_log'])
-        else:
-            return ''
+            try:
+                data = zlib.decompress(f.get('script_log'))
+            except:
+                data = ''
+        return data
 
     def get_name(self, nzo_id):
         t = (nzo_id,)
         if self.execute('SELECT name FROM history WHERE nzo_id=?', t):
-            return self.c.fetchone()['name']
+            return self.c.fetchone().get('name')
         else:
             return ''
 
 
+    def get_path(self, nzo_id):
+        t = (nzo_id,)
+        if self.execute('SELECT path FROM history WHERE nzo_id=?', t):
+            return self.c.fetchone().get('path')
+        else:
+            return ''
+
 def dict_factory(cursor, row):
     d = {}
     for idx, col in enumerate(cursor.description):
         d[col[0]] = row[idx]
     return d
 
-def build_history_info(nzo, storage='', path='', postproc_time=0, script_output='', script_line=''):
+def build_history_info(nzo, storage='', downpath='', postproc_time=0, script_output='', script_line=''):
     ''' Collects all the information needed for the database '''
 
-    path = decode_factory(path)
+    if not downpath:
+        downpath = nzo.downpath
+    path = decode_factory(downpath)
     storage = decode_factory(storage)
     script_line = decode_factory(script_line)
 
-    flagRepair, flagUnpack, flagDelete = nzo.get_repair_opts()
-    nzo_info = decode_factory(nzo.get_nzo_info())
+    flagRepair, flagUnpack, flagDelete = nzo.repair_opts
+    nzo_info = decode_factory(nzo.nzo_info)
 
     # Get the url and newzbin msgid
     report = decode_factory(nzo_info.get('msgid', ''))
@@ -290,22 +308,22 @@ def build_history_info(nzo, storage='', path='', postproc_time=0, script_output=
     else:
         url = decode_factory(nzo_info.get('url', ''))
 
-    #group = nzo.get_group()
+    #group = nzo.group
 
     completed = int(time.time())
-    name = decode_factory(nzo.get_original_dirname())
+    name = decode_factory(nzo.final_name)
 
-    nzb_name = decode_factory(nzo.get_filename())
-    category = decode_factory(nzo.get_cat())
+    nzb_name = decode_factory(nzo.filename)
+    category = decode_factory(nzo.cat)
     pps = ['','R','U','D']
     try:
         pp = pps[sabnzbd.opts_to_pp(flagRepair, flagUnpack, flagDelete)]
     except:
         pp = ''
-    script = decode_factory(nzo.get_script())
-    status = decode_factory(nzo.get_status())
-    nzo_id = nzo.get_nzo_id()
-    bytes = nzo.get_bytes_downloaded()
+    script = decode_factory(nzo.script)
+    status = decode_factory(nzo.status)
+    nzo_id = nzo.nzo_id
+    bytes = nzo.bytes_downloaded
 
     if script_output:
         # Compress the output of the script
@@ -316,13 +334,13 @@ def build_history_info(nzo, storage='', path='', postproc_time=0, script_output=
 
     download_time = decode_factory(nzo_info.get('download_time', 0))
 
-    downloaded = nzo.get_bytes_downloaded()
+    downloaded = nzo.bytes_downloaded
     completeness = 0
-    fail_message = decode_factory(nzo.get_fail_msg())
+    fail_message = decode_factory(nzo.fail_msg)
     url_info = nzo_info.get('more_info', '')
 
     # Get the dictionary containing the stages and their unpack process
-    stages = decode_factory(nzo.get_unpack_info())
+    stages = decode_factory(nzo.unpack_info)
     # Pack the ditionary up into a single string
     # Stage Name is seperated by ::: stage lines by ; and stages by \r\n
     lines = []
@@ -344,7 +362,7 @@ def unpack_history_info(item):
         try:
             lines = item['stage_log'].split('\r\n')
         except:
-            logging.error(T('error-badHisNewline@1'), unicoder(item['name']))
+            logging.error(T('Invalid stage logging in history for %s') + ' (\\r\\n)', unicoder(item['name']))
             logging.debug('Lines: %s', item['stage_log'])
             lines = []
         item['stage_log'] = []
@@ -361,7 +379,7 @@ def unpack_history_info(item):
             try:
                 logs = logs.split(';')
             except:
-                logging.error(T('error-badHisSemic@1'), unicoder(item['name']))
+                logging.error(T('Invalid stage logging in history for %s') + ' (;)', unicoder(item['name']))
                 logging.debug('Logs: %s', logs)
                 logs = []
             for log in logs:
@@ -397,11 +415,3 @@ def decode_factory(text):
     else:
         return text
 
-
-def find_monday():
-    last_monday = datetime.date.today()
-    minus_one_day = datetime.timedelta(days=1)
-    while last_monday.weekday() != MONDAY:
-        last_monday -= minus_one_day
-    return (last_monday.year, last_monday.month, last_monday.day, 0, 1, 1, 0, 0, 0)
-
