@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,7 +22,6 @@ sabnzbd.downloader - download engine
 import time
 import select
 import logging
-import datetime
 from threading import Thread, RLock
 from nntplib import NNTPPermanentError
 import socket
@@ -35,10 +34,8 @@ from sabnzbd.utils import osx
 from sabnzbd.constants import *
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-import sabnzbd.bpsmeter as bpsmeter
+from sabnzbd.bpsmeter import BPSMeter
 import sabnzbd.scheduler
-import sabnzbd.nzbqueue
-from sabnzbd.lang import T, Ta
 
 #------------------------------------------------------------------------------
 # Timeout penalty in minutes for each cause
@@ -53,154 +50,9 @@ _PENALTY_SHORT   = 1    # Minimal penalty when no_penalties is set
 TIMER_LOCK = RLock()
 
 #------------------------------------------------------------------------------
-# Wrapper functions
-
-__DOWNLOADER = None  # Global pointer to post-proc instance
-
-
-def init(paused):
-    global __DOWNLOADER
-    if __DOWNLOADER:
-        __DOWNLOADER.__init__(paused or __DOWNLOADER.paused)
-    else:
-        __DOWNLOADER = Downloader(paused)
-
-def start():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.start()
-
-
-def servers():
-    global __DOWNLOADER
-    if __DOWNLOADER: return __DOWNLOADER.servers
-
-def stop():
-    global __DOWNLOADER
-    CV.acquire()
-    try:
-        __DOWNLOADER.stop()
-    finally:
-        CV.notifyAll()
-        CV.release()
-    try:
-        __DOWNLOADER.join()
-    except:
-        pass
-
-def alive():
-    global __DOWNLOADER
-    if __DOWNLOADER:
-        return __DOWNLOADER.isAlive()
-    else:
-        return False
-
-def check():
-    global __DOWNLOADER
-    if __DOWNLOADER:
-        __DOWNLOADER.check_timers()
-
-#------------------------------------------------------------------------------
-
-@synchronized_CV
-def pause_downloader(save=True):
-    global __DOWNLOADER
-    if __DOWNLOADER:
-        __DOWNLOADER.pause()
-        if cfg.AUTODISCONNECT.get():
-            __DOWNLOADER.disconnect()
-        if save:
-            sabnzbd.save_state()
-
-@synchronized_CV
-def resume_downloader():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.resume()
-
-@synchronized_CV
-def delay_downloader():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.delay()
-
-@synchronized_CV
-def undelay_downloader():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.undelay()
-
-@synchronized_CV
-def idle_downloader():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.wait_postproc()
-
-@synchronized_CV
-def unidle_downloader():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.resume_postproc()
-
-@synchronized_CV
-def limit_speed(value):
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.limit_speed(int(value))
-    logging.info("Bandwidth limit set to %s", value)
-
-def update_server(oldserver, newserver):
-    global __DOWNLOADER
-    try:
-        CV.acquire()
-        try:
-            __DOWNLOADER.init_server(oldserver, newserver)
-        finally:
-            CV.notifyAll()
-            CV.release()
-    except:
-        logging.exception("Error accessing DOWNLOADER?")
-
-@synchronized_CV
-def set_paused(state):
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.paused = state
-
-@synchronized_CV
-def unblock(server):
-    global __DOWNLOADER
-    if __DOWNLOADER: return __DOWNLOADER.unblock(server)
-
-@synchronized_CV
-def unblock_all():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.unblock_all()
-
-@synchronized_CV
-def wakeup():
-    # Just let the decorator rattle the semaphore
-    pass
-
-
-#------------------------------------------------------------------------------
-
-def paused():
-    global __DOWNLOADER
-    if __DOWNLOADER: return __DOWNLOADER.paused
-
-def get_limit():
-    global __DOWNLOADER
-    if __DOWNLOADER: return __DOWNLOADER.get_limit()
-
-def disconnect():
-    global __DOWNLOADER
-    if __DOWNLOADER: __DOWNLOADER.disconnect()
-
-def delayed():
-    global __DOWNLOADER
-    if __DOWNLOADER: return __DOWNLOADER.delayed
-
-def active_primaries():
-    global __DOWNLOADER
-    if __DOWNLOADER: return __DOWNLOADER.active_primaries()
-
-#------------------------------------------------------------------------------
-class Server:
+class Server(object):
     def __init__(self, id, host, port, timeout, threads, fillserver, ssl, username = None,
-                 password = None, optional=False):
+                 password = None, optional=False, retention=0):
         self.id = id
         self.newid = None
         self.restart = False
@@ -211,6 +63,7 @@ class Server:
         self.fillserver = fillserver
         self.ssl = ssl
         self.optional = optional
+        self.retention = retention
 
         self.username = username
         self.password = password
@@ -247,7 +100,11 @@ class Server:
 #------------------------------------------------------------------------------
 
 class Downloader(Thread):
-    def __init__(self, paused = False):
+    """ Singleton Downloader Thread
+    """
+    do = None
+
+    def __init__(self, paused=False):
         Thread.__init__(self)
 
         logging.debug("Initializing downloader/decoder")
@@ -256,8 +113,8 @@ class Downloader(Thread):
         self.paused = paused
 
         #used for throttling bandwidth and scheduling bandwidth changes
-        self.bandwidth_limit = cfg.BANDWIDTH_LIMIT.get()
-        cfg.BANDWIDTH_LIMIT.callback(self.speed_set)
+        self.bandwidth_limit = cfg.bandwidth_limit()
+        cfg.bandwidth_limit.callback(self.speed_set)
 
         # Used for reducing speed
         self.delayed = False
@@ -282,7 +139,7 @@ class Downloader(Thread):
             self.init_server(None, server)
 
         self.decoder = Decoder(self.servers)
-
+        Downloader.do = self
 
     def init_server(self, oldserver, newserver):
         """ Setup or re-setup single server
@@ -297,17 +154,18 @@ class Downloader(Thread):
         servers = config.get_servers()
         if newserver in servers:
             srv = servers[newserver]
-            enabled = srv.enable.get()
-            host = srv.host.get()
-            port = srv.port.get()
-            timeout = srv.timeout.get()
-            threads = srv.connections.get()
-            fillserver = srv.fillserver.get()
+            enabled = srv.enable()
+            host = srv.host()
+            port = srv.port()
+            timeout = srv.timeout()
+            threads = srv.connections()
+            fillserver = srv.fillserver()
             primary = enabled and (not fillserver) and (threads > 0)
-            ssl = srv.ssl.get() and sabnzbd.newswrapper.HAVE_SSL
-            username = srv.username.get()
-            password = srv.password.get()
-            optional = srv.optional.get()
+            ssl = srv.ssl() and sabnzbd.newswrapper.HAVE_SSL
+            username = srv.username()
+            password = srv.password()
+            optional = srv.optional()
+            retention = float(srv.retention() * 24 * 3600) # days ==> seconds
             create = True
 
         if oldserver:
@@ -322,58 +180,75 @@ class Downloader(Thread):
 
         if create and enabled and host and port and threads:
             self.servers.append(Server(newserver, host, port, timeout, threads, fillserver, ssl,
-                                            username, password, optional))
+                                            username, password, optional, retention))
 
         return primary
 
-    def stop(self):
-        self.shutdown = True
-        osx.sendGrowlMsg("SABnzbd",T('grwl-shutdown-begin-msg'),osx.NOTIFICATION['startup'])
+    @synchronized_CV
+    def set_paused_state(self, state):
+        """ Set Downloader to specified paused state """
+        self.paused = state
 
+    @synchronized_CV
     def resume(self):
         logging.info("Resuming")
         self.paused = False
 
-    def pause(self):
-        logging.info("Pausing")
-        osx.sendGrowlMsg("SABnzbd",T('grwl-paused-msg'),osx.NOTIFICATION['download'])
-        self.paused = True
-        if self.is_paused():
-            bpsmeter.method.reset()
+    @synchronized_CV
+    def pause(self, save=True):
+        """ Pause the downloader, optionally saving admin
+        """
+        if not self.paused:
+            self.paused = True
+            logging.info("Pausing")
+            osx.sendGrowlMsg("SABnzbd",T('Paused'),osx.NOTIFICATION['download'])
+            if self.is_paused():
+                BPSMeter.do.reset()
+            if cfg.autodisconnect():
+                self.disconnect()
+            if save:
+                sabnzbd.save_state()
 
+    @synchronized_CV
     def delay(self):
         logging.debug("Delaying")
         self.delayed = True
 
+    @synchronized_CV
     def undelay(self):
         logging.debug("Undelaying")
         self.delayed = False
 
-    def wait_postproc(self):
+    @synchronized_CV
+    def wait_for_postproc(self):
         logging.info("Waiting for post-processing to finish")
         self.postproc = True
 
-    def resume_postproc(self):
+    @synchronized_CV
+    def resume_from_postproc(self):
         logging.info("Post-processing finished, resuming download")
         self.postproc = False
 
     def disconnect(self):
         self.force_disconnect = True
 
+    @synchronized_CV
     def limit_speed(self, value):
-        self.bandwidth_limit = value
+        self.bandwidth_limit = int(value)
+        logging.info("Bandwidth limit set to %s", value)
 
     def get_limit(self):
         return self.bandwidth_limit
 
     def speed_set(self):
-        self.bandwidth_limit = cfg.BANDWIDTH_LIMIT.get()
+        self.bandwidth_limit = cfg.bandwidth_limit()
 
     def is_paused(self):
+        from sabnzbd.nzbqueue import NzbQueue
         if not self.paused:
             return False
         else:
-            if sabnzbd.nzbqueue.has_forced_items():
+            if NzbQueue.do.has_forced_items():
                 return False
             else:
                 return True
@@ -386,22 +261,25 @@ class Downloader(Thread):
         return False
 
     def maybe_block_server(self, server):
+        from sabnzbd.nzbqueue import NzbQueue
         if server.optional and server.active and (server.bad_cons/server.threads) > 3:
             # Optional and active server had too many problems,
             # disable it now and send a re-enable plan to the scheduler
             server.bad_cons = 0
             server.active = False
-            server.errormsg = T('warn-ignoreServer@2') % ('', _PENALTY_TIMEOUT)
-            logging.warning(Ta('warn-ignoreServer@2'), server.id, _PENALTY_TIMEOUT)
+            server.errormsg = T('Server %s will be ignored for %s minutes') % ('', _PENALTY_TIMEOUT)
+            logging.warning(Ta('Server %s will be ignored for %s minutes'), server.id, _PENALTY_TIMEOUT)
             self.plan_server(server.id, _PENALTY_TIMEOUT)
-            sabnzbd.nzbqueue.reset_all_try_lists()
+            NzbQueue.do.reset_all_try_lists()
 
 
     def run(self):
+        from sabnzbd.nzbqueue import NzbQueue
         self.decoder.start()
 
         while 1:
             for server in self.servers:
+                assert isinstance(server, Server)
                 for nw in server.busy_threads[:]:
                     if nw.nntp.error_msg or (nw.timeout and time.time() > nw.timeout):
                         if nw.nntp.error_msg:
@@ -418,20 +296,22 @@ class Downloader(Thread):
                         if newid:
                             self.init_server(None, newid)
                         self.__restart -= 1
-                        sabnzbd.nzbqueue.reset_all_try_lists()
+                        NzbQueue.do.reset_all_try_lists()
                         # Have to leave this loop, because we removed element
                         break
                     else:
                         # Restart pending, don't add new articles
                         continue
 
+                assert isinstance(server, Server)
                 if not server.idle_threads or server.restart or self.is_paused() or self.shutdown or self.delayed or self.postproc:
                     continue
 
-                if not (server.active and sabnzbd.nzbqueue.has_articles_for(server)):
+                if not (server.active and NzbQueue.do.has_articles_for(server)):
                     continue
 
                 for nw in server.idle_threads[:]:
+                    assert isinstance(nw, NewsWrapper)
                     if nw.timeout:
                         if time.time() < nw.timeout:
                             continue
@@ -446,31 +326,36 @@ class Downloader(Thread):
                         request_server_info(server)
                         break
 
-                    article = sabnzbd.nzbqueue.get_article(server)
+                    article = NzbQueue.do.get_article(server)
 
                     if not article:
                         break
 
+                    if server.retention and article.nzf.nzo.avg_stamp < time.time() - server.retention:
+                        # Article too old for the server, treat as missing
+                        if sabnzbd.LOG_ALL:
+                            logging.debug('Article %s too old for %s:%s', article.article, server.host, server.port)
+                        self.decoder.decode(article, None)
+                        break
+
+                    server.idle_threads.remove(nw)
+                    server.busy_threads.append(nw)
+
+                    nw.article = article
+
+                    if nw.connected:
+                        self.__request_article(nw)
                     else:
-                        server.idle_threads.remove(nw)
-                        server.busy_threads.append(nw)
-
-                        nw.article = article
-
-                        if nw.connected:
-                            self.__request_article(nw)
-                        else:
-                            try:
-                                logging.info("%s@%s:%s: Initiating connection",
-                                                  nw.thrdnum, server.host, server.port)
-                                nw.init_connect()
-                                self.write_fds[nw.nntp.sock.fileno()] = nw
-                            except:
-                                logging.error(Ta('error-noInit@3'),
-                                                  nw.thrdnum, server.host,
-                                                  server.port)
-                                logging.info("Traceback: ", exc_info = True)
-                                self.__reset_nw(nw, "failed to initialize")
+                        try:
+                            logging.info("%s@%s:%s: Initiating connection",
+                                              nw.thrdnum, server.host, server.port)
+                            nw.init_connect(self.write_fds)
+                        except:
+                            logging.error(Ta('Failed to initialize %s@%s:%s'),
+                                              nw.thrdnum, server.host,
+                                              server.port)
+                            logging.info("Traceback: ", exc_info = True)
+                            self.__reset_nw(nw, "failed to initialize")
 
             # Exit-point
             if self.shutdown:
@@ -510,12 +395,12 @@ class Downloader(Thread):
             else:
                 read, write, error = ([], [], [])
 
-                bpsmeter.method.reset()
+                BPSMeter.do.reset()
 
                 time.sleep(1.0)
 
                 CV.acquire()
-                while (not sabnzbd.nzbqueue.has_articles() or self.is_paused() or self.delayed or self.postproc) and not \
+                while (NzbQueue.do.is_empty() or self.is_paused() or self.delayed or self.postproc) and not \
                        self.shutdown and not self.__restart:
                     CV.wait()
                 CV.release()
@@ -534,7 +419,7 @@ class Downloader(Thread):
                     self.write_fds.pop(fileno)
 
             if not read:
-                bpsmeter.method.update(0)
+                BPSMeter.do.update()
                 continue
 
             for selected in read:
@@ -551,7 +436,7 @@ class Downloader(Thread):
                     bytes, done, skip = (0, False, False)
 
                 if skip:
-                    bpsmeter.method.update(0)
+                    BPSMeter.do.update()
                     continue
 
                 if bytes < 1:
@@ -560,26 +445,26 @@ class Downloader(Thread):
 
                 else:
                     if self.bandwidth_limit:
-                        bps = bpsmeter.method.get_bps()
+                        bps = BPSMeter.do.get_bps()
                         bps += bytes
                         limit = self.bandwidth_limit * 1024
                         if bps > limit:
-                            while bpsmeter.method.get_bps() > limit:
+                            while BPSMeter.do.get_bps() > limit:
                                 time.sleep(0.05)
-                                bpsmeter.method.update(0)
-                    bpsmeter.method.update(bytes)
+                                BPSMeter.do.update()
+                    BPSMeter.do.update(server.id, bytes)
 
                     if nzo:
-                        nzo.update_bytes(bytes)
-                        nzo.update_avg_kbs(bpsmeter.method.get_bps())
+                        nzo.bytes_downloaded += bytes
+                        nzo.update_avg_kbs(BPSMeter.do.get_bps())
 
                 if len(nw.lines) == 1:
                     code = nw.lines[0][:3]
-                    if not nw.connected:
+                    if not nw.connected or code == '480':
                         done = False
 
                         try:
-                            nw.finish_connect()
+                            nw.finish_connect(code)
                             if sabnzbd.LOG_ALL:
                                 logging.debug("%s@%s:%s last message -> %s", nw.thrdnum, nw.server.host,
                                               nw.server.port, nw.lines[0])
@@ -598,51 +483,51 @@ class Downloader(Thread):
                                 # Too many connections: remove this thread and reduce thread-setting for server
                                 # Plan to go back to the full number after a penalty timeout
                                 if server.active:
-                                    server.errormsg = Ta('error-serverTooMany@2') % ('', display_msg)
-                                    logging.error(Ta('error-serverTooMany@2'), server.host, server.port)
+                                    server.errormsg = Ta('Too many connections to server %s:%s') % ('', display_msg)
+                                    logging.error(Ta('Too many connections to server %s:%s'), server.host, server.port)
                                     self.__reset_nw(nw, None, warn=False, destroy=True, quit=True)
                                     self.plan_server(server.id, _PENALTY_TOOMANY)
                                     server.threads -= 1
                             elif ecode in ('502', '481') and clues_too_many_ip(msg):
                                 # Account sharing?
                                 if server.active:
-                                    server.errormsg = Ta('error-accountSharing') + display_msg
+                                    server.errormsg = Ta('Probable account sharing') + display_msg
                                     name = ' (%s:%s)' % (server.host, server.port)
-                                    logging.error(Ta('error-accountSharing') + name)
+                                    logging.error(Ta('Probable account sharing') + name)
                                     penalty = _PENALTY_SHARE
                             elif ecode in ('481', '482', '381') or (ecode == '502' and clues_login(msg)):
                                 # Cannot login, block this server
                                 if server.active:
-                                    server.errormsg = Ta('error-serverLogin@1') % display_msg
-                                    logging.error(Ta('error-serverLogin@1'), '%s:%s' % (server.host, server.port))
+                                    server.errormsg = Ta('Failed login for server %s') % display_msg
+                                    logging.error(Ta('Failed login for server %s'), '%s:%s' % (server.host, server.port))
                                 penalty = _PENALTY_PERM
                                 block = True
                             elif ecode == '502':
                                 # Cannot connect (other reasons), block this server
                                 if server.active:
-                                    server.errormsg = Ta('warn-noConnectServer@2') % ('', display_msg)
-                                    logging.warning(Ta('warn-noConnectServer@2'), '%s:%s' % (server.host, server.port), msg)
+                                    server.errormsg = Ta('Cannot connect to server %s [%s]') % ('', display_msg)
+                                    logging.warning(Ta('Cannot connect to server %s [%s]'), '%s:%s' % (server.host, server.port), msg)
                                 penalty = _PENALTY_502
                                 block = True
                             else:
                                 # Unknown error, just keep trying
                                 if server.active:
-                                    server.errormsg = Ta('error-serverNoConn@2') % ('', display_msg)
-                                    logging.error(Ta('error-serverNoConn@2'),  '%s:%s' % (server.host, server.port), msg)
+                                    server.errormsg = Ta('Cannot connect to server %s [%s]') % ('', display_msg)
+                                    logging.error(Ta('Cannot connect to server %s [%s]'),  '%s:%s' % (server.host, server.port), msg)
                                     penalty = _PENALTY_UNKNOWN
                             if block or (penalty and server.optional):
                                 if server.active:
                                     server.active = False
-                                    if (not server.optional) and cfg.NO_PENALTIES.get():
+                                    if (not server.optional) and cfg.no_penalties():
                                         penalty = _PENALTY_SHORT
                                     if penalty and (block or server.optional):
                                         logging.info('Server %s ignored for %s minutes', server.id, penalty)
                                         self.plan_server(server.id, penalty)
-                                    sabnzbd.nzbqueue.reset_all_try_lists()
+                                    NzbQueue.do.reset_all_try_lists()
                                 self.__reset_nw(nw, None, warn=False, quit=True)
                             continue
                         except:
-                            logging.error(Ta('error-serverFailed@4'),
+                            logging.error(Ta('Connecting %s@%s:%s failed, message=%s'),
                                               nw.thrdnum,
                                               nw.server.host, nw.server.port, nw.lines[0])
                             # No reset-warning needed, above logging is sufficient
@@ -659,7 +544,7 @@ class Downloader(Thread):
 
                         logging.debug("group command ok -> %s",
                                       nw.lines)
-                        nw.group = nw.article.nzf.nzo.get_group()
+                        nw.group = nw.article.nzf.nzo.group
                         nw.lines = []
                         nw.data = ''
                         self.__request_article(nw)
@@ -669,17 +554,17 @@ class Downloader(Thread):
                         nw.lines = None
 
                         logging.info('Thread %s@%s:%s: Article ' + \
-                                        '%s missing',
+                                        '%s missing (error=%s)',
                                         nw.thrdnum, nw.server.host,
-                                        nw.server.port, article.article)
+                                        nw.server.port, article.article, code)
 
                     elif code == '480':
                         if server.active:
                             server.active = False
-                            server.errormsg = T('error-serverCred@1') % ''
+                            server.errormsg = T('Server %s requires user/password') % ''
                             self.plan_server(server.id, 0)
-                            sabnzbd.nzbqueue.reset_all_try_lists()
-                        msg = T('error-serverCred@1') % ('%s:%s' % (nw.server.host, nw.server.port))
+                            NzbQueue.do.reset_all_try_lists()
+                        msg = T('Server %s requires user/password') % ('%s:%s' % (nw.server.host, nw.server.port))
                         self.__reset_nw(nw, msg, quit=True)
 
                 if done:
@@ -703,6 +588,7 @@ class Downloader(Thread):
         return None
 
     def __reset_nw(self, nw, errormsg, warn=True, wait=True, destroy=False, quit=False):
+        from sabnzbd.nzbqueue import NzbQueue
         server = nw.server
         article = nw.article
         fileno = None
@@ -741,7 +627,7 @@ class Downloader(Thread):
             nzo = nzf.nzo
 
             ## Allow all servers to iterate over each nzo/nzf again ##
-            sabnzbd.nzbqueue.reset_try_lists(nzf, nzo)
+            NzbQueue.do.reset_try_lists(nzf, nzo)
 
         if destroy:
             nw.terminate(quit=quit)
@@ -750,16 +636,16 @@ class Downloader(Thread):
 
     def __request_article(self, nw):
         try:
-            if cfg.SEND_GROUP.get() and nw.article.nzf.nzo.get_group() != nw.group:
-                group = nw.article.nzf.nzo.get_group()
+            if cfg.send_group() and nw.article.nzf.nzo.group != nw.group:
+                group = nw.article.nzf.nzo.group
                 if sabnzbd.LOG_ALL:
-                    logging.info('Thread %s@%s:%s: GROUP <%s>', nw.thrdnum, nw.server.host,
-                                 nw.server.port, group)
+                    logging.debug('Thread %s@%s:%s: GROUP <%s>', nw.thrdnum, nw.server.host,
+                                   nw.server.port, group)
                 nw.send_group(group)
             else:
                 if sabnzbd.LOG_ALL:
-                    logging.info('Thread %s@%s:%s: BODY %s', nw.thrdnum, nw.server.host,
-                                 nw.server.port, nw.article.article)
+                    logging.debug('Thread %s@%s:%s: BODY %s', nw.thrdnum, nw.server.host,
+                                  nw.server.port, nw.article.article)
                 nw.body()
 
             fileno = nw.nntp.sock.fileno()
@@ -800,6 +686,7 @@ class Downloader(Thread):
                 del self._timers[server_id]
                 self.init_server(server_id, server_id)
 
+    @synchronized_CV
     @synchronized(TIMER_LOCK)
     def unblock(self, server_id):
         # Remove timer
@@ -814,10 +701,12 @@ class Downloader(Thread):
                 self.init_server(server_id, server_id)
                 break
 
+
     def unblock_all(self):
         for server_id in self._timers.keys():
             self.unblock(server_id)
 
+    @synchronized_CV
     @synchronized(TIMER_LOCK)
     def check_timers(self):
         """ Make sure every server without a non-expired timer is active """
@@ -837,6 +726,34 @@ class Downloader(Thread):
                     logging.debug('Forcing activation of server %s', server.id)
                     self.init_server(server.id, server.id)
 
+    @synchronized_CV
+    def update_server(self, oldserver, newserver):
+        self.init_server(oldserver, newserver)
+
+    @synchronized_CV
+    def wakeup(self):
+        """ Just rattle the semaphore
+        """
+        pass
+
+    def stop(self):
+        self.shutdown = True
+        osx.sendGrowlMsg("SABnzbd",T('Shutting down'),osx.NOTIFICATION['startup'])
+
+
+def stop():
+    CV.acquire()
+    try:
+        Downloader.do.stop()
+    finally:
+        CV.notifyAll()
+        CV.release()
+    try:
+        Downloader.do.join()
+    except:
+        pass
+
+
 #------------------------------------------------------------------------------
 def clues_login(text):
     """ Check for any "failed login" clues in the response code
