@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2010 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,10 +29,9 @@ import time
 import glob
 from sabnzbd.constants import *
 import sabnzbd
-from sabnzbd.misc import to_units, split_host
-from sabnzbd.codecs import EmailFilter
+from sabnzbd.misc import to_units, split_host, time_format
+from sabnzbd.encoding import EmailFilter, latin1
 import sabnzbd.cfg as cfg
-from sabnzbd.lang import T, Ta
 
 
 ################################################################################
@@ -40,17 +39,20 @@ from sabnzbd.lang import T, Ta
 #
 #
 ################################################################################
-def send(message):
+def send(message, recipient):
     """ Send message if message non-empty and email-parms are set """
+    def errormsg(msg):
+        logging.error(latin1(msg))
+        return msg
+
     if not message.strip('\n\r\t '):
         return "Skipped empty message"
 
-    if cfg.EMAIL_SERVER.get() and cfg.EMAIL_TO.get() and cfg.EMAIL_FROM.get():
+    if cfg.email_server() and recipient and cfg.email_from():
 
         message = _prepare_message(message)
 
-        failure = T('error-mailSend')
-        server, port = split_host(cfg.EMAIL_SERVER.get())
+        server, port = split_host(cfg.email_server())
         if not port:
             port = 25
 
@@ -73,11 +75,9 @@ def send(message):
                     mailconn = smtplib.SMTP(server, port)
                     mailconn.ehlo()
                 except:
-                    logging.error(Ta('error-mailNoConn'))
-                    return failure
+                    return errormsg(T('Failed to connect to mail server'))
             else:
-                logging.error(Ta('error-mailNoConn'))
-                return failure
+                return errormsg(T('Failed to connect to mail server'))
 
         # TLS support
         if mailconn.ehlo_resp:
@@ -89,53 +89,106 @@ def send(message):
                     mailconn.starttls()
                     mailconn.ehlo()
                 except:
-                    logging.error(Ta('error-mailTLS'))
-                    return failure
+                    return errormsg(T('Failed to initiate TLS connection'))
 
         # Authentication
-        if (cfg.EMAIL_ACCOUNT.get() != "") and (cfg.EMAIL_PWD.get() != ""):
+        if (cfg.email_account() != "") and (cfg.email_pwd() != ""):
             try:
-                mailconn.login(cfg.EMAIL_ACCOUNT.get(), cfg.EMAIL_PWD.get())
+                mailconn.login(cfg.email_account(), cfg.email_pwd())
             except:
-                logging.error(Ta('error-mailAuth'))
-                return failure
+                return errormsg(T('Failed to authenticate to mail server'))
 
         try:
-            mailconn.sendmail(cfg.EMAIL_FROM.get(), cfg.EMAIL_TO.get(), message)
+            mailconn.sendmail(cfg.email_from(), cfg.email_to(), message)
+            msg = None
+        except smtplib.SMTPHeloError:
+            msg = errormsg('The server didn\'t reply properly to the helo greeting.')
+        except smtplib.SMTPRecipientsRefused:
+            msg = errormsg('The server rejected ALL recipients (no mail was sent).')
+        except smtplib.SMTPSenderRefused:
+            msg = errormsg('The server didn\'t accept the from_addr.')
+        except smtplib.SMTPDataError:
+            msg = errormsg('The server replied with an unexpected error code (other than a refusal of a recipient).')
         except:
-            logging.error(Ta('error-mailSend'))
-            return failure
+            msg = errormsg(T('Failed to send e-mail'))
 
         try:
             mailconn.close()
         except:
-            logging.warning(Ta('warn-noEmailClose'))
+            errormsg(T('Failed to close mail connection'))
 
-        logging.info("Notification e-mail succesfully sent")
-        return T('msg-emailOK')
+        if msg:
+            return msg
+        else:
+            logging.info("Notification e-mail succesfully sent")
+            return T('Email succeeded')
 
 
+def get_email_date():
+    """ Return un-localized date string for the Date: field
+    """
+    # Get locale indepedant date/time string: "Sun May 22 20:15:12 2011"
+    day, month, dayno, hms, year = time.asctime(time.gmtime()).split()
+    return '%s, %s %s %s %s +0000' % (day, dayno, month, year, hms)
+
 
 ################################################################################
-# EMAIL_ENDJOB
+# email_endjob
 #
 #
 ################################################################################
 from Cheetah.Template import Template
 
+def send_with_template(prefix, parm):
+    """ Send an email using template """
+
+    parm['from'] = cfg.email_from()
+    parm['date'] = get_email_date()
+
+    lst = []
+    path = cfg.email_dir.get_path()
+    if path and os.path.exists(path):
+        try:
+            lst = glob.glob(os.path.join(path, '%s-*.tmpl' % prefix))
+        except:
+            logging.error(Ta('Cannot find email templates in %s'), path)
+    else:
+        path = os.path.join(sabnzbd.DIR_PROG, DEF_EMAIL_TMPL)
+        tpath = os.path.join(path, '%s-%s.tmpl' % (prefix, cfg.language()))
+        if os.path.exists(tpath):
+            lst = [tpath]
+        else:
+            lst = [os.path.join(path, '%s-en.tmpl' % prefix)]
+
+    ret = T('No email templates found')
+    for temp in lst:
+        if os.access(temp, os.R_OK):
+            source = _decode_file(temp)
+            if source:
+                for recipient in cfg.email_to():
+                    parm['to'] = recipient
+                    message = Template(source=source,
+                                        searchList=[parm],
+                                        filter=EmailFilter,
+                                        compilerSettings={'directiveStartToken': '<!--#',
+                                                          'directiveEndToken': '#-->'})
+                    ret = send(message.respond(), recipient)
+                    del message
+            else:
+                ret = T('Invalid encoding of email template %s') % temp
+    return ret
+
+
 def endjob(filename, msgid, cat, status, path, bytes, stages, script, script_output, script_ret):
-    """ Send email using templates """
+    """ Send end-of-job email """
 
     # Translate the stage names
     xstages = {}
     for stage in stages:
-        xstages[T('stage-'+stage.lower())] = stages[stage]
+        xstages[Tx('stage-'+stage.lower())] = stages[stage]
 
     parm = {}
     parm['status'] = status
-    parm['to'] = cfg.EMAIL_TO.get_string()
-    parm['from'] = cfg.EMAIL_FROM.get()
-    parm['date'] = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime())
     parm['name'] = filename
     parm['path'] = path
     parm['msgid'] = str(msgid)
@@ -145,34 +198,16 @@ def endjob(filename, msgid, cat, status, path, bytes, stages, script, script_out
     parm['script_ret'] = script_ret
     parm['cat'] = cat
     parm['size'] = "%sB" % to_units(bytes)
-    parm['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
+    parm['end_time'] = time.strftime(time_format('%Y-%m-%d %H:%M:%S'), time.localtime(time.time()))
 
-    lst = []
-    path = cfg.EMAIL_DIR.get_path()
-    if path and os.path.exists(path):
-        try:
-            lst = glob.glob(os.path.join(path, '*.tmpl'))
-        except:
-            logging.error(Ta('error-mailTempl@1'), path)
-    else:
-        path = os.path.join(sabnzbd.DIR_PROG, DEF_LANGUAGE)
-        path = os.path.join(path, 'email-%s.tmpl' % cfg.LANGUAGE.get())
-        if os.path.exists(path):
-            lst = [path]
+    return send_with_template('email', parm)
 
-    ret = "No templates found"
-    for temp in lst:
-        if os.access(temp, os.R_OK):
-            source = _decode_file(temp)
-            message = Template(source=source,
-                                searchList=[parm],
-                                filter=EmailFilter,
-                                compilerSettings={'directiveStartToken': '<!--#',
-                                                  'directiveEndToken': '#-->'})
-            ret = send(message.respond())
-            del message
-    return ret
 
+def rss_mail(feed, jobs):
+    """ Send notification email containing list of files """
+
+    parm = {'amount' : len(jobs), 'feed' : feed, 'jobs' : jobs}
+    return send_with_template('rss', parm)
 
 
 ################################################################################
@@ -183,8 +218,18 @@ def endjob(filename, msgid, cat, status, path, bytes, stages, script, script_out
 def diskfull():
     """ Send email about disk full, no templates """
 
-    if cfg.EMAIL_FULL.get():
-        return send(T('email-full@2') % (cfg.EMAIL_TO.get_string(), cfg.EMAIL_FROM.get()))
+    if cfg.email_full():
+        return send(T('''To: %s
+From: %s
+Date: %s
+Subject: SABnzbd reports Disk Full
+
+Hi,
+
+SABnzbd has stopped downloading, because the disk is almost full.
+Please make room and resume SABnzbd manually.
+
+''') % (cfg.email_to.get_string(), cfg.email_from(), get_email_date()), cfg.email_to())
     else:
         return ""
 
@@ -205,24 +250,22 @@ def _decode_file(path):
     source = fp.read()
     fp.close()
 
-    return source.decode(encoding)
+    try:
+        return source.decode(encoding)
+    except:
+        return ''
 
 
 ################################################################################
-try:
-    from email.message import Message
-    from email.header import Header
-    from email.encoders import encode_quopri
-except:
-    # Python 2.4 uses different names
-    from email.Message import Message
-    from email.Header import Header
-    from email.Encoders import encode_quopri
+from email.message import Message
+from email.header import Header
+from email.encoders import encode_quopri
 
 RE_HEADER = re.compile(r'^([^:]+):(.*)')
 
 def _prepare_message(txt):
-    """ Do the proper message encodings
+    """ Apply the proper encoding to all email fields.
+        The body will be Latin-1, the headers will be 'quopri'd when necessary.
     """
     def plain(val):
         """ Return True when val is plain ASCII """
@@ -236,7 +279,6 @@ def _prepare_message(txt):
     code = 'ISO-8859-1'
 
     msg = Message()
-    msg.set_charset(code)
     payload = []
     body = False
     header = False
@@ -261,11 +303,8 @@ def _prepare_message(txt):
 
     msg.set_payload('\n'.join(payload), code)
 
-    # Prevent double header (because it will be added again by encode_quopri)
-    try:
-        del msg['Content-Transfer-Encoding']
-    except:
-        pass
+    # Check for proper encoding, else call it explicitly
+    if not msg.has_key('Content-Transfer-Encoding'):
+        encode_quopri(msg)
 
-    encode_quopri(msg)
     return msg.as_string()
