@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,7 +28,6 @@ import logging
 
 import sabnzbd
 from sabnzbd.constants import *
-from sabnzbd.lang import Ta
 
 try:
     from OpenSSL import SSL
@@ -40,7 +39,7 @@ try:
 except ImportError:
     _ssl = None
     HAVE_SSL = False
-    
+
     # Dummy class so this exception is ignored by clients without ssl installed
     class WantReadError(Exception):
         def __init__(self, value):
@@ -73,7 +72,7 @@ def _retrieve_info(server):
     else:
         server.bad_cons = 0
     (server.info, server.request) = (info, False)
-    sabnzbd.downloader.wakeup()
+    sabnzbd.downloader.Downloader.do.wakeup()
 
 
 def request_server_info(server):
@@ -104,7 +103,8 @@ def GetServerParms(host, port):
             return None
 
 
-def con(sock, host, port, sslenabled, nntp):
+def con(sock, host, port, sslenabled, write_fds, nntp):
+    assert isinstance(nntp, NNTP)
     try:
         sock.connect((host, port))
         sock.setblocking(0)
@@ -115,6 +115,13 @@ def con(sock, host, port, sslenabled, nntp):
                     break
                 except WantReadError:
                     select.select([sock], [], [], 1.0)
+
+        # Now it's safe to add the socket to the list of active sockets.
+        # 'write_fds' is an attribute of the Downloader singleton.
+        # This direct access is needed to prevent multi-threading sync problems.
+        if write_fds is not None:
+            write_fds[sock.fileno()] = nntp.nw
+
     except socket.error, e:
         try:
             # socket.error can either return a string or a tuple
@@ -133,11 +140,12 @@ def con(sock, host, port, sslenabled, nntp):
     except _ssl.Error, e:
         nntp.error(e)
 
-class NNTP:
-    def __init__(self, host, port, info, sslenabled, nntp, user=None, password=None, block=False):
+class NNTP(object):
+    def __init__(self, host, port, info, sslenabled, nw, user=None, password=None, block=False, write_fds=None):
+        assert isinstance(nw, NewsWrapper)
         self.host = host
         self.port = port
-        self.nntp = nntp
+        self.nw = nw
         self.blocking = block
         self.error_msg = None
         if not info:
@@ -150,7 +158,7 @@ class NNTP:
 
         if sslenabled and _ssl:
             # Some users benefit from SSLv2 not being capped.
-            ssl_type = sabnzbd.cfg.SSL_TYPE.get()
+            ssl_type = sabnzbd.cfg.ssl_type.get()
             if ssl_type == 'v2':
                 ctx = _ssl.Context(_ssl.SSLv2_METHOD)
             elif ssl_type == 'v3':
@@ -160,7 +168,7 @@ class NNTP:
 
             self.sock = SSLConnection(ctx, socket.socket(af, socktype, proto))
         elif sslenabled and not _ssl:
-            logging.error(Ta('error-noSSL'))
+            logging.error(Ta('Error importing OpenSSL module. Connecting with NON-SSL'))
             self.sock = socket.socket(af, socktype, proto)
         else:
             self.sock = socket.socket(af, socktype, proto)
@@ -168,8 +176,8 @@ class NNTP:
         try:
             # Windows must do the connection in a seperate thread due to non-blocking issues
             # If the server wants to be blocked (for testing) then use the linux route
-            if sabnzbd.WIN32 and not block:
-                Thread(target=con, args=(self.sock, self.host, self.port, sslenabled, self)).start()
+            if not block:
+                Thread(target=con, args=(self.sock, self.host, self.port, sslenabled, write_fds, self)).start()
             else:
                 # if blocking (server test) only wait for 4 seconds during connect until timeout
                 if block:
@@ -207,15 +215,15 @@ class NNTP:
         if 'SSL23_GET_SERVER_HELLO' in str(error):
             error = 'This server does not allow SSL on this port'
         msg = "Failed to connect: %s" % (str(error))
-        msg = "%s %s@%s:%s" % (msg, self.nntp.thrdnum, self.host, self.port)
+        msg = "%s %s@%s:%s" % (msg, self.nw.thrdnum, self.host, self.port)
         self.error_msg = msg
         if self.blocking:
             raise socket.error(errno.ECONNREFUSED, msg)
         else:
             logging.info(msg)
-            self.nntp.server.warning = msg
+            self.nw.server.warning = msg
 
-class NewsWrapper:
+class NewsWrapper(object):
     def __init__(self, server, thrdnum, block=False):
         self.server = server
         self.thrdnum = thrdnum
@@ -238,30 +246,39 @@ class NewsWrapper:
 
         self.user_ok = False
         self.pass_ok = False
+        self.force_login = False
 
-    def init_connect(self):
+    def init_connect(self, write_fds):
         self.nntp = NNTP(self.server.host, self.server.port, self.server.info, self.server.ssl, self,
-                         self.server.username, self.server.password, self.blocking)
+                         self.server.username, self.server.password, self.blocking, write_fds)
         self.recv = self.nntp.sock.recv
 
         self.timeout = time.time() + self.server.timeout
 
-    def finish_connect(self):
-        if not self.server.username or not self.server.password:
+    def finish_connect(self, code):
+        if not (self.server.username or self.server.password or self.force_login):
             self.connected = True
             self.user_sent = True
             self.user_ok = True
             self.pass_sent = True
             self.pass_ok = True
 
-        if self.lines and self.lines[0][:3] == '400':
+        if code == '480':
+            self.force_login = True
+            self.connected = False
+            self.user_sent = False
+            self.user_ok = False
+            self.pass_sent = False
+            self.pass_ok = False
+
+        if code == '400':
             raise NNTPPermanentError(self.lines[0])
         elif not self.user_sent:
             command = 'authinfo user %s\r\n' % (self.server.username)
             self.nntp.sock.sendall(command)
             self.user_sent = True
         elif not self.user_ok:
-            if self.lines[0][:3] == '381':
+            if code == '381':
                 self.user_ok = True
 
         if self.user_ok and not self.pass_sent:
@@ -269,7 +286,7 @@ class NewsWrapper:
             self.nntp.sock.sendall(command)
             self.pass_sent = True
         elif self.user_ok and not self.pass_ok:
-            if self.lines[0][:3] != '281':
+            if code != '281':
                 # Assume that login failed (code 481 or other)
                 raise NNTPPermanentError(self.lines[0])
             else:
@@ -288,6 +305,8 @@ class NewsWrapper:
         self.nntp.sock.sendall(command)
 
     def recv_chunk(self, block=False):
+        """ Receive data, return #bytes, done, skip
+        """
         self.timeout = time.time() + self.server.timeout
         while 1:
             try:
@@ -354,7 +373,7 @@ class NewsWrapper:
         del self.nntp
 
 
-class SSLConnection:
+class SSLConnection(object):
     def __init__(self, *args):
         self._ssl_conn = apply(_ssl.Connection, args)
         self._lock = _RLock()
