@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -35,96 +35,14 @@ except:
 from threading import *
 
 import sabnzbd
-from sabnzbd.constants import *
+from sabnzbd.constants import BOOKMARK_FILE_NAME
 from sabnzbd.decorators import synchronized
-from sabnzbd.misc import cat_to_opts, sanitize_foldername, bad_fetch, CatConvert
-from sabnzbd.codecs import name_fixer
+from sabnzbd.misc import cat_to_opts, sanitize_foldername, bad_fetch, cat_convert
+from sabnzbd.encoding import name_fixer
 import sabnzbd.newswrapper
-import sabnzbd.nzbqueue
 import sabnzbd.cfg as cfg
-from sabnzbd.lang import T, Ta
 from sabnzbd.utils import osx
 
-################################################################################
-# BOOKMARK Wrappers
-################################################################################
-
-__BOOKMARKS = None
-
-def bookmarks_init():
-    global __BOOKMARKS
-    if not __BOOKMARKS:
-        __BOOKMARKS = Bookmarks()
-
-
-def bookmarks_save():
-    global __BOOKMARKS
-    if __BOOKMARKS:
-        __BOOKMARKS.save()
-
-
-def getBookmarksNow():
-    global __BOOKMARKS
-    if __BOOKMARKS:
-        __BOOKMARKS.run()
-
-
-def getBookmarksList():
-    global __BOOKMARKS
-    if __BOOKMARKS:
-        return __BOOKMARKS.bookmarksList()
-
-
-def delete_bookmark(msgid):
-    global __BOOKMARKS
-    if __BOOKMARKS and cfg.NEWZBIN_UNBOOKMARK.get():
-        __BOOKMARKS.del_bookmark(msgid)
-
-
-################################################################################
-# Msgid Grabber Wrappers
-################################################################################
-__MSGIDGRABBER = None
-
-def init_grabber():
-    global __MSGIDGRABBER
-    if __MSGIDGRABBER:
-        __MSGIDGRABBER.__init__()
-    else:
-        __MSGIDGRABBER = MSGIDGrabber()
-
-
-def start_grabber():
-    global __MSGIDGRABBER
-    if __MSGIDGRABBER:
-        logging.debug('Starting msgidgrabber')
-        __MSGIDGRABBER.start()
-
-
-def stop_grabber():
-    global __MSGIDGRABBER
-    if __MSGIDGRABBER:
-        logging.debug('Stopping msgidgrabber')
-        __MSGIDGRABBER.stop()
-        try:
-            __MSGIDGRABBER.join()
-        except:
-            pass
-
-
-def grab(msgid, future_nzo):
-    global __MSGIDGRABBER
-    if __MSGIDGRABBER:
-        __MSGIDGRABBER.grab(msgid, future_nzo)
-
-def alive():
-    global __MSGIDGRABBER
-    if __MSGIDGRABBER:
-        return __MSGIDGRABBER.isAlive()
-    else:
-        return False
-
-
 
 ################################################################################
 # DirectNZB support
@@ -149,12 +67,16 @@ def _access_ok():
 
 class MSGIDGrabber(Thread):
     """ Thread for msgid-grabber queue """
+    do = None # Link to instance of thread
+
     def __init__(self):
+        from sabnzbd.nzbqueue import NzbQueue
         Thread.__init__(self)
         self.queue = Queue.Queue()
-        for tup in sabnzbd.nzbqueue.get_msgids():
+        for tup in NzbQueue.do.get_msgids():
             self.queue.put(tup)
         self.shutdown = False
+        MSGIDGrabber.do = self
 
     def grab(self, msgid, nzo):
         logging.debug("Adding msgid %s to the queue", msgid)
@@ -167,6 +89,7 @@ class MSGIDGrabber(Thread):
 
     def run(self):
         """ Process the queue (including waits and retries) """
+        from sabnzbd.nzbqueue import NzbQueue
         def sleeper(delay):
             for n in range(delay):
                 if not self.shutdown:
@@ -185,24 +108,24 @@ class MSGIDGrabber(Thread):
             if filename and data:
                 filename = name_fixer(filename)
 
-                _r, _u, _d = nzo.get_repair_opts()
-                pp = sabnzbd.opts_to_pp(_r, _u, _d)
-                script = nzo.get_script()
-                cat = nzo.get_cat()
-                if not cat:
-                    cat = CatConvert(newzbin_cat)
+                pp = nzo.pp
+                script = nzo.script
+                cat = nzo.cat
+                if cat == '*' or not cat:
+                    cat = cat_convert(newzbin_cat)
 
-                priority = nzo.get_priority()
-                nzbname = nzo.get_dirname_rename()
+                priority = nzo.priority
+                nzbname = nzo.custom_name
 
                 cat, pp, script, priority = cat_to_opts(cat, pp, script, priority)
 
                 try:
                     sabnzbd.nzbqueue.insert_future_nzo(nzo, filename, msgid, data, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname, nzo_info=nzo_info)
                 except:
-                    logging.error(Ta('error-nbUpdate@1'), msgid)
+                    logging.error(Ta('Failed to update newzbin job %s'), msgid)
                     logging.info("Traceback: ", exc_info = True)
-                    sabnzbd.nzbqueue.remove_nzo(nzo.nzo_id, False)
+
+                    NzbQueue.do.remove(nzo.nzo_id, False)
                 msgid = None
             else:
                 if filename:
@@ -212,7 +135,7 @@ class MSGIDGrabber(Thread):
                     bad_fetch(nzo, msgid, msg=nzo_info, retry=True)
                     msgid = None
 
-            osx.sendGrowlMsg(T('grwl-nzbadd-title'),filename,osx.NOTIFICATION['download'])
+            osx.sendGrowlMsg(T('NZB added to queue'),filename,osx.NOTIFICATION['download'])
 
             # Keep some distance between the grabs
             sleeper(5)
@@ -233,12 +156,12 @@ def _grabnzb(msgid):
 
     # Connect to Newzbin
     try:
-        if _HAVE_SSL and cfg.NEWZBIN_HTTPS.get():
+        if _HAVE_SSL:
             conn = httplib.HTTPSConnection('www.newzbin.com')
         else:
             conn = httplib.HTTPConnection('www.newzbin.com')
 
-        postdata = { 'username': cfg.USERNAME_NEWZBIN.get(), 'password': cfg.PASSWORD_NEWZBIN.get(), 'reportid': msgid }
+        postdata = { 'username': cfg.newzbin_username(), 'password': cfg.newzbin_password(), 'reportid': msgid }
         postdata = urllib.urlencode(postdata)
 
         headers['Content-type'] = 'application/x-www-form-urlencoded'
@@ -264,7 +187,7 @@ def _grabnzb(msgid):
         # Only some reports will generate a moreinfo header
         pass
     if not (rcode or rtext):
-        logging.error(T('error-nbProtocol'))
+        logging.error(T('Newzbin server changed its protocol'))
         return None, None, None, None
 
     # Official return codes:
@@ -298,26 +221,26 @@ def _grabnzb(msgid):
         return int(wait+1), None, None, None
 
     if rcode in ('402'):
-        msg = Ta('warn-nbCredit')
+        msg = Ta('You have no credit on your Newzbin account')
         return None, None, None, msg
 
     if rcode in ('401'):
-        msg = Ta('warn-nbNoAuth')
+        msg = Ta('Unauthorised, check your newzbin username/password')
         return None, None, None, msg
 
     if rcode in ('400', '404'):
-        msg = Ta('error-nbReport@1') % msgid
+        msg = Ta('Newzbin report %s not found') % msgid
         return None, None, None, msg
 
     if rcode != '200':
-        msg = Ta('error-nbUnkownError@2') % (rcode, rtext)
+        msg = Ta('Newzbin gives undocumented error code (%s, %s)') % (rcode, rtext)
         return None, None, None, msg
 
     # Process data
     report_name = response.getheader('X-DNZB-Name')
     report_cat  = response.getheader('X-DNZB-Category')
     if not (report_name and report_cat):
-        msg = Ta('error-nbInfo@1') %  msgid
+        msg = Ta('Newzbin server fails to give info for %s') %  msgid
         return None, None, None, msg
 
     # sanitize report_name
@@ -336,14 +259,17 @@ def _grabnzb(msgid):
 ################################################################################
 BOOK_LOCK = Lock()
 
-class Bookmarks:
+class Bookmarks(object):
     """ Get list of bookmarks from www.newzbin.com
     """
+    do = None # Link to instance
+
     def __init__(self):
-        self.bookmarks = sabnzbd.load_data(BOOKMARK_FILE_NAME)
+        self.bookmarks = sabnzbd.load_admin(BOOKMARK_FILE_NAME)
         if not self.bookmarks:
             self.bookmarks = []
         self.__busy = False
+        Bookmarks.do = self
 
     @synchronized(BOOK_LOCK)
     def run(self, delete=None):
@@ -352,18 +278,18 @@ class Bookmarks:
 
         # Connect to Newzbin
         try:
-            if _HAVE_SSL and cfg.NEWZBIN_HTTPS.get():
+            if _HAVE_SSL:
                 conn = httplib.HTTPSConnection('www.newzbin.com')
             else:
                 conn = httplib.HTTPConnection('www.newzbin.com')
 
             if delete:
                 logging.debug('Trying to delete Newzbin bookmark %s', delete)
-                postdata = { 'username': cfg.USERNAME_NEWZBIN.get(), 'password': cfg.PASSWORD_NEWZBIN.get(), 'action': 'delete', \
+                postdata = { 'username': cfg.newzbin_username(), 'password': cfg.newzbin_password(), 'action': 'delete', \
                              'reportids' : delete }
             else:
                 logging.info('Fetching Newzbin bookmarks')
-                postdata = { 'username': cfg.USERNAME_NEWZBIN.get(), 'password': cfg.PASSWORD_NEWZBIN.get(), 'action': 'fetch'}
+                postdata = { 'username': cfg.newzbin_username(), 'password': cfg.newzbin_password(), 'action': 'fetch'}
             postdata = urllib.urlencode(postdata)
 
             headers['Content-type'] = 'application/x-www-form-urlencoded'
@@ -398,18 +324,20 @@ class Bookmarks:
         if rcode == '204':
             logging.debug("No bookmarks set")
         elif rcode in ('401', '403'):
-            logging.warning(Ta('warn-nbNoAuth'))
+            logging.warning(Ta('Unauthorised, check your newzbin username/password'))
         elif rcode in ('402'):
-            logging.warning(Ta('warn-nbCredit'))
+            logging.warning(Ta('You have no credit on your Newzbin account'))
         elif rcode in ('500', '503'):
             _warn_user('Newzbin has a server problem (%s).' % rcode)
         elif rcode == '200':
             if delete:
                 if data.startswith('1'):
                     logging.info('Deleted newzbin bookmark %s', delete)
-                    self.bookmarks.remove(delete)
+                    if delete in self.bookmarks:
+                        self.bookmarks.remove(delete)
                 else:
-                    logging.warning(Ta('warn-nbNoDelBM@1'), delete)
+                    if delete in self.bookmarks:
+                        logging.warning(Ta('Could not delete newzbin bookmark %s'), delete)
             else:
                 for line in data.split('\n'):
                     try:
@@ -421,18 +349,21 @@ class Bookmarks:
                         logging.info("Found new bookmarked msgid %s (%s)", msgid, text)
                         sabnzbd.add_msgid(int(msgid), None, None, priority=None)
         else:
-            logging.error(Ta('error-nbUnkownError@1'), rcode)
+            logging.error(Ta('Newzbin gives undocumented error code (%s)'), rcode)
 
+        self._save()
         self.__busy = False
 
     @synchronized(BOOK_LOCK)
     def save(self):
-        sabnzbd.save_data(self.bookmarks, BOOKMARK_FILE_NAME)
+        self._save()
+
+    def _save(self):
+        sabnzbd.save_admin(self.bookmarks, BOOKMARK_FILE_NAME)
 
     def bookmarksList(self):
         return self.bookmarks
 
     def del_bookmark(self, msgid):
-        msgid = str(msgid)
-        if msgid in self.bookmarks:
-            self.run(msgid)
+        if cfg.newzbin_unbookmark():
+            self.run(str(msgid))
