@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,78 +27,158 @@ import datetime
 import sabnzbd
 from sabnzbd.trylist import TryList
 from sabnzbd.nzbstuff import NzbObject
-from sabnzbd.misc import panic_queue, exit_sab, sanitize_foldername, cat_to_opts
+from sabnzbd.misc import panic_queue, exit_sab, cat_to_opts, \
+                         get_admin_path, remove_all, globber
 import sabnzbd.database as database
-from sabnzbd.decorators import *
-from sabnzbd.constants import *
+from sabnzbd.decorators import NZBQUEUE_LOCK, synchronized, synchronized_CV
+from sabnzbd.constants import QUEUE_FILE_NAME, QUEUE_VERSION, FUTURE_Q_FOLDER, JOB_ADMIN, \
+                              LOW_PRIORITY, NORMAL_PRIORITY, HIGH_PRIORITY, TOP_PRIORITY, \
+                              PNFO_BYTES_FIELD, PNFO_BYTES_LEFT_FIELD
 import sabnzbd.cfg as cfg
-import sabnzbd.articlecache
+from sabnzbd.articlecache import ArticleCache
 import sabnzbd.downloader
-import sabnzbd.assembler
-from sabnzbd.lang import T, Ta
+from sabnzbd.assembler import Assembler, file_has_articles
 from sabnzbd.utils import osx
-
-
-def DeleteLog(name):
-    if name:
-        name = name.replace('.nzb', '.log')
-        try:
-            os.remove(os.path.join(os.path.dirname(sabnzbd.LOGFILE), name))
-        except:
-            pass
+from sabnzbd.encoding import latin1, platform_encode
+from sabnzbd.bpsmeter import BPSMeter
 
 #-------------------------------------------------------------------------------
 
 class NzbQueue(TryList):
+    """ Singleton NzbQueue """
+    do = None
+
     def __init__(self):
         TryList.__init__(self)
 
-        self.__downloaded_items = []
-
-
-        self.__top_only = cfg.TOP_ONLY.get()
+        self.__top_only = False #cfg.top_only()
         self.__top_nzo = None
 
         self.__nzo_list = []
         self.__nzo_table = {}
 
-        self.__auto_sort = cfg.AUTO_SORT.get()
+        self.__auto_sort = cfg.auto_sort()
+        NzbQueue.do = self
 
+    def read_queue(self, repair):
+        """ Read queue from disk, supporting repair modes
+            0 = no repairs
+            1 = use existing queue, add missing "incomplete" folders
+            2 = Discard all queue admin, reconstruct from "incomplete" folders
+        """
         nzo_ids = []
-
-        data = sabnzbd.load_data(QUEUE_FILE_NAME, remove = False)
-
-        if data:
-            try:
-                queue_vers, nzo_ids, self.__downloaded_items = data
-                if not queue_vers == QUEUE_VERSION:
-                    logging.error(Ta('error-qBad'))
-                    self.__downloaded_items = []
+        if repair < 2:
+            # Read the queue from the saved files
+            data = sabnzbd.load_admin(QUEUE_FILE_NAME)
+            if data:
+                try:
+                    queue_vers, nzo_ids, dummy = data
+                    if not queue_vers == QUEUE_VERSION:
+                        nzo_ids = []
+                        logging.error(Ta('Incompatible queuefile found, cannot proceed'))
+                        if not repair:
+                            panic_queue(os.path.join(cfg.cache_dir.get_path(), QUEUE_FILE_NAME))
+                            exit_sab(2)
+                except ValueError:
                     nzo_ids = []
-                    panic_queue(os.path.join(cfg.CACHE_DIR.get_path(),QUEUE_FILE_NAME))
-                    exit_sab(2)
-            except ValueError:
-                logging.error(Ta('error-qCorruptFile@1'),
-                              os.path.join(cfg.CACHE_DIR.get_path(), QUEUE_FILE_NAME))
+                    logging.error(Ta('Error loading %s, corrupt file detected'),
+                                  os.path.join(cfg.cache_dir.get_path(), QUEUE_FILE_NAME))
+                    if not repair:
+                        return
+
+        # First handle jobs in the queue file
+        folders = []
+        for nzo_id in nzo_ids:
+            folder, _id = os.path.split(nzo_id)
+            # Try as normal job
+            path = get_admin_path(bool(folder), folder, False)
+            nzo = sabnzbd.load_data(_id, path, remove=False)
+            if not nzo:
+                # Try as future job
+                path = get_admin_path(bool(folder), folder, True)
+                nzo = sabnzbd.load_data(_id, path)
+            if nzo:
+                self.add(nzo, save=False, quiet=True)
+                folders.append(folder)
+
+        # Scan for any folders in "incomplete" that are not yet in the queue
+        if repair:
+            self.scan_jobs(not folders)
+            # Handle any lost future jobs
+            for nzo_id in globber(os.path.join(cfg.admin_dir.get_path(), FUTURE_Q_FOLDER)):
+                if nzo_id not in self.__nzo_table:
+                    self.add(nzo, save=False)
+
+
+    def scan_jobs(self, all=False, action=True):
+        """ Scan "incomplete" for mssing folders,
+            'all' is True: Include active folders
+            'action' is True, do the recovery action
+            returns list of orphaned folders
+        """
+        result = []
+        # Folders from the download queue
+        if all:
+            registered = []
+        else:
+            registered = [nzo.work_name for nzo in self.__nzo_list]
+
+        # Retryable folders from History
+        items = sabnzbd.proxy_build_history()[0]
+        # Anything waiting or active or retryable is a known item
+        registered.extend([platform_encode(os.path.basename(item['path'])) \
+                           for item in items if item['retry'] or item['loaded'] or item['status'] == 'Queued'])
+
+        # Repair unregistered folders
+        for folder in globber(cfg.download_dir.get_path()):
+            if os.path.isdir(folder) and os.path.basename(folder) not in registered:
+                if action:
+                    logging.info('Repairing job %s', folder)
+                    self.repair_job(folder)
+                result.append(os.path.basename(folder))
+            else:
+                if action:
+                    logging.info('Skipping repair for job %s', folder)
+        return result
+
+
+    def repair_job(self, folder, new_nzb=None):
+        """ Reconstruct admin for a single job folder, optionally with new NZB """
+        name = os.path.basename(folder)
+        path = os.path.join(folder, JOB_ADMIN)
+        if new_nzb is None or not new_nzb.filename:
+            filename = globber(path, '*.gz')
+            if len(filename) > 0:
+                logging.debug('Repair job %s by reparsing stored NZB', latin1(name))
+                sabnzbd.add_nzbfile(filename[0], pp=None, script=None, cat=None, priority=None, nzbname=name, reuse=True)
+            else:
+                logging.debug('Repair job %s without stored NZB', latin1(name))
+                nzo = NzbObject(name, 0, pp=None, script=None, nzb='', cat=None, priority=None, nzbname=name, reuse=True)
+                self.add(nzo)
+        else:
+            remove_all(path, '*.gz')
+            logging.debug('Repair job %s without new NZB (%s)', latin1(name), latin1(new_nzb.filename))
+            sabnzbd.add_nzbfile(new_nzb, pp=None, script=None, cat=None, priority=None, nzbname=name, reuse=True)
 
-            for nzo_id in nzo_ids:
-                nzo = sabnzbd.load_data(nzo_id, remove = False)
-                if nzo:
-                    self.add(nzo, save = False)
 
     @synchronized(NZBQUEUE_LOCK)
-    def save(self):
-        """ Save queue """
+    def save(self, save_nzo=None):
+        """ Save queue, all nzo's or just the specified one """
         logging.info("Saving queue")
 
         nzo_ids = []
         # Aggregate nzo_ids and save each nzo
         for nzo in self.__nzo_list:
-            nzo_ids.append(nzo.nzo_id)
-            sabnzbd.save_data(nzo, nzo.nzo_id)
+            if nzo.new_caching:
+                nzo_ids.append(os.path.join(nzo.work_name, nzo.nzo_id))
+            else:
+                nzo_ids.append(nzo.nzo_id)
+            if save_nzo is None or nzo is save_nzo:
+                sabnzbd.save_data(nzo, nzo.nzo_id, nzo.workpath)
+                if not nzo.futuretype:
+                    nzo.save_attribs()
 
-        sabnzbd.save_data((QUEUE_VERSION, nzo_ids,
-                           self.__downloaded_items), QUEUE_FILE_NAME)
+        sabnzbd.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
 
     @synchronized(NZBQUEUE_LOCK)
     def set_top_only(self, value):
@@ -114,30 +194,32 @@ class NzbQueue(TryList):
     @synchronized(NZBQUEUE_LOCK)
     def insert_future(self, future, filename, msgid, data, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None, nzo_info=None):
         """ Refresh a placeholder nzo with an actual nzo """
+        assert isinstance(future, NzbObject)
         if nzo_info is None:
             nzo_info = {}
         nzo_id = future.nzo_id
         if nzo_id in self.__nzo_table:
             try:
+                sabnzbd.remove_data(nzo_id, future.workpath)
                 logging.info("Regenerating item: %s", nzo_id)
-                r, u, d = future.get_repair_opts()
+                r, u, d = future.repair_opts
                 if not r is None:
                     pp = sabnzbd.opts_to_pp(r, u, d)
-                scr = future.get_script()
+                scr = future.script
                 if scr is None:
                     scr = script
-                categ = future.get_cat()
+                categ = future.cat
                 if categ is None:
                     categ = cat
                 categ, pp, script, priority = cat_to_opts(categ, pp, script, priority)
 
                 # Remember old priority
-                old_prio = future.get_priority()
+                old_prio = future.priority
 
                 try:
                     future.__init__(filename, msgid, pp, scr, nzb=data, futuretype=False, cat=categ, priority=priority, nzbname=nzbname, nzo_info=nzo_info)
                     future.nzo_id = nzo_id
-                    self.save()
+                    self.save(future)
                 except ValueError:
                     self.remove(nzo_id, False)
                 except TypeError:
@@ -145,7 +227,7 @@ class NzbQueue(TryList):
 
                 # Make sure the priority is changed now that we know the category
                 if old_prio != priority:
-                    future.set_priority(None)
+                    future.priority = None
                 self.set_priority(future.nzo_id, priority)
 
                 if self.__auto_sort:
@@ -153,7 +235,7 @@ class NzbQueue(TryList):
 
                 self.reset_try_list()
             except:
-                logging.error(Ta('error-qAdd@1'), nzo_id)
+                logging.error(Ta('Error while adding %s, removing'), nzo_id)
                 logging.info("Traceback: ", exc_info = True)
                 self.remove(nzo_id, False)
         else:
@@ -161,24 +243,30 @@ class NzbQueue(TryList):
                          nzo_id)
 
     @synchronized(NZBQUEUE_LOCK)
-    def change_opts(self, nzo_id, pp):
-        if nzo_id in self.__nzo_table:
-            self.__nzo_table[nzo_id].set_opts(pp)
+    def change_opts(self, nzo_ids, pp):
+        for nzo_id in [item.strip() for item in nzo_ids.split(',')]:
+            if nzo_id in self.__nzo_table:
+                self.__nzo_table[nzo_id].set_pp(pp)
 
     @synchronized(NZBQUEUE_LOCK)
-    def change_script(self, nzo_id, script):
-        if nzo_id in self.__nzo_table:
-            self.__nzo_table[nzo_id].set_script(script)
+    def change_script(self, nzo_ids, script):
+        for nzo_id in [item.strip() for item in nzo_ids.split(',')]:
+            if nzo_id in self.__nzo_table:
+                self.__nzo_table[nzo_id].script = script
 
     @synchronized(NZBQUEUE_LOCK)
-    def change_cat(self, nzo_id, cat):
-        if nzo_id in self.__nzo_table:
-            self.__nzo_table[nzo_id].set_cat(cat)
+    def change_cat(self, nzo_ids, cat):
+        for nzo_id in [item.strip() for item in nzo_ids.split(',')]:
+            if nzo_id in self.__nzo_table:
+                nzo = self.__nzo_table[nzo_id]
+                nzo.cat, pp, nzo.script, prio = cat_to_opts(cat)
+                nzo.set_pp(pp)
+                self.set_priority(nzo_id, prio)
 
     @synchronized(NZBQUEUE_LOCK)
     def change_name(self, nzo_id, name):
         if nzo_id in self.__nzo_table:
-            self.__nzo_table[nzo_id].set_name(name)
+            self.__nzo_table[nzo_id].set_final_name_pw(name)
 
     @synchronized(NZBQUEUE_LOCK)
     def get_nzo(self, nzo_id):
@@ -188,20 +276,26 @@ class NzbQueue(TryList):
             return None
 
     @synchronized(NZBQUEUE_LOCK)
-    def add(self, nzo, save=True):
+    def add(self, nzo, save=True, quiet=False):
+        assert isinstance(nzo, NzbObject)
         sabnzbd.QUEUECOMPLETEACTION_GO = False
 
+        if not nzo.nzo_id:
+            nzo.nzo_id = sabnzbd.get_new_id('nzo', nzo.workpath, self.__nzo_table)
+
+        # If no files are to be downloaded anymore, send to postproc
+        if not nzo.files and not nzo.futuretype:
+            sabnzbd.remove_data(nzo.nzo_id, nzo.workpath)
+            sabnzbd.proxy_postproc(nzo)
+            return
+
         # Reset try_lists
         nzo.reset_try_list()
         self.reset_try_list()
 
-
-        if not nzo.nzo_id:
-            nzo.nzo_id = sabnzbd.get_new_id('nzo')
-
         if nzo.nzo_id:
             nzo.deleted = False
-            priority = nzo.get_priority()
+            priority = nzo.priority
             self.__nzo_table[nzo.nzo_id] = nzo
             if priority == TOP_PRIORITY:
                 #A top priority item (usually a completed download fetching pars)
@@ -218,7 +312,7 @@ class NzbQueue(TryList):
                     pos = 0
                     added = False
                     for position in self.__nzo_list:
-                        if position.get_priority() < priority:
+                        if position.priority < priority:
                             self.__nzo_list.insert(pos, nzo)
                             added=True
                             break
@@ -231,21 +325,23 @@ class NzbQueue(TryList):
                     #if the queue is empty then simple append the item to the bottom
                     self.__nzo_list.append(nzo)
             if save:
-                self.save()
+                self.save(nzo)
 
-            if nzo.get_status() not in ('Fetching',):
-                osx.sendGrowlMsg(T('grwl-nzbadd-title'),nzo.get_filename(),osx.NOTIFICATION['download'])
+            if not (quiet or nzo.status in ('Fetching',)):
+                osx.sendGrowlMsg(T('NZB added to queue'), nzo.filename, osx.NOTIFICATION['download'])
 
         if self.__auto_sort:
             self.sort_by_avg_age()
 
     @synchronized(NZBQUEUE_LOCK)
-    def remove(self, nzo_id, add_to_history = True, unload=False, save=True, cleanup=True):
+    def remove(self, nzo_id, add_to_history = True, save=True, cleanup=True, keep_basic=False, del_files=False):
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table.pop(nzo_id)
             nzo.deleted = True
             self.__nzo_list.remove(nzo)
 
+            sabnzbd.remove_data(nzo_id, nzo.workpath)
+
             if add_to_history:
                 # Create the history DB instance
                 history_db = database.get_history_handle()
@@ -255,18 +351,18 @@ class NzbQueue(TryList):
                 history_db.close()
 
             elif cleanup:
-                self.cleanup_nzo(nzo)
+                self.cleanup_nzo(nzo, keep_basic, del_files)
 
-            sabnzbd.remove_data(nzo_id)
             if save:
-                self.save()
+                self.save(nzo)
 
 
     @synchronized(NZBQUEUE_LOCK)
-    def remove_multiple(self, nzo_ids):
+    def remove_multiple(self, nzo_ids, del_files=False):
         for nzo_id in nzo_ids:
-            self.remove(nzo_id, add_to_history = False, save = False)
-        self.save()
+            self.remove(nzo_id, add_to_history = False, save = False, keep_basic=not del_files, del_files=del_files)
+        # Save with invalid nzo_id, to that only queue file is saved
+        self.save('x')
 
     @synchronized(NZBQUEUE_LOCK)
     def remove_all(self):
@@ -277,8 +373,8 @@ class NzbQueue(TryList):
             nzo = self.__nzo_table.pop(nzo_id)
             nzo.deleted = True
             self.__nzo_list.remove(nzo)
+            sabnzbd.remove_data(nzo_id, nzo.workpath)
             self.cleanup_nzo(nzo)
-            sabnzbd.remove_data(nzo_id)
         del lst
         self.save()
 
@@ -291,7 +387,10 @@ class NzbQueue(TryList):
             if nzf:
                 post_done = nzo.remove_nzf(nzf)
                 if post_done:
-                    self.remove(nzo_id, add_to_history = False)
+                    keep_basic = nzo.finished_files
+                    if keep_basic:
+                        sabnzbd.proxy_postproc(nzo)
+                    self.remove(nzo_id, add_to_history = False, keep_basic=keep_basic)
 
 
     @synchronized(NZBQUEUE_LOCK)
@@ -303,7 +402,7 @@ class NzbQueue(TryList):
     def pause_nzo(self, nzo_id):
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
-            nzo.pause_nzo()
+            nzo.pause()
             logging.debug("Paused nzo: %s", nzo_id)
 
     @synchronized(NZBQUEUE_LOCK)
@@ -315,7 +414,7 @@ class NzbQueue(TryList):
     def resume_nzo(self, nzo_id):
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
-            nzo.resume_nzo()
+            nzo.resume()
             nzo.reset_all_try_lists()
             logging.debug("Resumed nzo: %s", nzo_id)
         self.reset_try_list()
@@ -336,20 +435,20 @@ class NzbQueue(TryList):
             return (-1, 0)
 
         #get the priorities of the two items
-        nzo1_priority = nzo1.get_priority()
-        nzo2_priority = nzo2.get_priority()
+        nzo1_priority = nzo1.priority
+        nzo2_priority = nzo2.priority
         try:
             #get the item id of the item below to use in priority changing
             item_id_3 = self.__nzo_list[i+1].nzo_id
             #if there is an item below the id1 and id2 then we need that too
             #to determine whether to change the priority
             nzo3 = self.__nzo_table[item_id_3]
-            nzo3_priority = nzo3.get_priority()
+            nzo3_priority = nzo3.priority
             #if id1 is surrounded by items of a different priority then change it's pririty to match
             if nzo2_priority != nzo1_priority and nzo3_priority != nzo1_priority or nzo2_priority > nzo1_priority:
-                nzo1.set_priority(nzo2_priority)
+                nzo1.priority = nzo2_priority
         except:
-            nzo1.set_priority(nzo2_priority)
+            nzo1.priority = nzo2_priority
         item_id_pos1 = -1
         item_id_pos2 = -1
         for i in xrange(len(self.__nzo_list)):
@@ -361,9 +460,9 @@ class NzbQueue(TryList):
                 item = self.__nzo_list[item_id_pos1]
                 del self.__nzo_list[item_id_pos1]
                 self.__nzo_list.insert(item_id_pos2, item)
-                return (item_id_pos2, nzo1.get_priority())
+                return (item_id_pos2, nzo1.priority)
         # If moving failed/no movement took place
-        return (-1, nzo1.get_priority())
+        return (-1, nzo1.priority)
 
     @synchronized(NZBQUEUE_LOCK)
     def get_position(self, nzb_id):
@@ -409,7 +508,7 @@ class NzbQueue(TryList):
 
 
     @synchronized(NZBQUEUE_LOCK)
-    def sort_queue(self, field, reverse=False):
+    def sort_queue(self, field, reverse=None):
         if isinstance(reverse, str):
             if reverse.lower() == 'desc':
                 reverse = True
@@ -426,9 +525,7 @@ class NzbQueue(TryList):
         else:
             logging.debug("Sort: %s not recognised", field)
 
-
-    @synchronized(NZBQUEUE_LOCK)
-    def set_priority(self, nzo_id, priority):
+    def __set_priority(self, nzo_id, priority):
         """ Sets the priority on the nzo and places it in the queue at the approrioate position """
         try:
             priority = int(priority)
@@ -443,10 +540,10 @@ class NzbQueue(TryList):
                     break
 
             # Don't change priority and order if priority is the same as asked
-            if priority == self.__nzo_list[nzo_id_pos1].get_priority():
+            if priority == self.__nzo_list[nzo_id_pos1].priority:
                 return nzo_id_pos1
 
-            nzo.set_priority(priority)
+            nzo.priority = priority
 
             if nzo_id_pos1 != -1:
                 del self.__nzo_list[nzo_id_pos1]
@@ -467,12 +564,12 @@ class NzbQueue(TryList):
                         p = 0
                         added = False
                         for position in self.__nzo_list:
-                            if position.get_priority() < priority:
+                            if position.priority < priority:
                                 self.__nzo_list.insert(p, nzo)
                                 pos = p
-                                added=True
+                                added = True
                                 break
-                            p+=1
+                            p += 1
                         if not added:
                             #if there are no other items classed as a lower priority
                             #then it will be added to the bottom of the queue
@@ -488,25 +585,15 @@ class NzbQueue(TryList):
             return -1
 
     @synchronized(NZBQUEUE_LOCK)
-    def set_priority_multiple(self, nzo_ids, priority):
+    def set_priority(self, nzo_ids, priority):
         try:
             n = -1
-            for nzo_id in nzo_ids:
-                n = self.set_priority(nzo_id, priority)
+            for nzo_id in [item.strip() for item in nzo_ids.split(',')]:
+                n = self.__set_priority(nzo_id, priority)
             return n
         except:
             return -1
 
-    @synchronized(NZBQUEUE_LOCK)
-    def set_original_dirname(self, nzo_id, name):
-        try:
-            if name:
-                nzo = self.__nzo_table[nzo_id]
-                name = sanitize_foldername(name)
-                nzo.set_original_dirname(name)
-        except:
-            pass
-
     @synchronized(NZBQUEUE_LOCK)
     def reset_try_lists(self, nzf = None, nzo = None):
         if nzf:
@@ -530,7 +617,7 @@ class NzbQueue(TryList):
         elif self.__top_only:
             for nzo in self.__nzo_list:
                 # Ignore any items that are in a paused or grabbing state
-                if nzo.get_status() not in ('Paused', 'Grabbing'):
+                if nzo.status not in ('Paused', 'Grabbing'):
                     return not nzo.server_in_try_list(server)
         else:
             return not self.server_in_try_list(server)
@@ -540,7 +627,7 @@ class NzbQueue(TryList):
         ''' Check if the queue contains any Forced
         Priority items to download while paused '''
         for nzo in self.__nzo_list:
-            if nzo.get_priority() == TOP_PRIORITY and nzo.get_status() not in ('Paused', 'Grabbing'):
+            if nzo.priority == TOP_PRIORITY and nzo.status not in ('Paused', 'Grabbing'):
                 return True
         return False
 
@@ -549,7 +636,7 @@ class NzbQueue(TryList):
         if self.__top_only:
             if self.__nzo_list:
                 for nzo in self.__nzo_list:
-                    if nzo.get_status() not in ('Paused', 'Grabbing'):
+                    if nzo.status not in ('Paused', 'Grabbing'):
                         article = nzo.get_article(server)
                         if article:
                             return article
@@ -557,7 +644,7 @@ class NzbQueue(TryList):
         else:
             for nzo in self.__nzo_list:
                 # Don't try to get an article if server is in try_list of nzo
-                if not nzo.server_in_try_list(server) and nzo.get_status() not in ('Paused', 'Grabbing'):
+                if not nzo.server_in_try_list(server) and nzo.status not in ('Paused', 'Grabbing'):
                     article = nzo.get_article(server)
                     if article:
                         return article
@@ -576,35 +663,35 @@ class NzbQueue(TryList):
 
         file_done, post_done, reset = nzo.remove_article(article)
 
-        filename = nzf.get_filename()
+        filename = nzf.filename
 
         if reset:
             self.reset_try_list()
 
         if file_done:
-            if nzo.extra3 is None or time.time() > nzo.extra3:
-                sabnzbd.save_data(nzo, nzo.nzo_id)
-                if nzo.extra4 is None:
-                    nzo.extra3 = None
+            if nzo.next_save is None or time.time() > nzo.next_save:
+                sabnzbd.save_data(nzo, nzo.nzo_id, nzo.workpath)
+                BPSMeter.do.save()
+                if nzo.save_timeout is None:
+                    nzo.next_save = None
                 else:
-                    nzo.extra3 = time.time() + nzo.extra4
+                    nzo.next_save = time.time() + nzo.save_timeout
 
-            _type = nzf.get_type()
+            _type = nzf.type
 
             # Only start decoding if we have a filename and type
             if filename and _type:
-                sabnzbd.assembler.process((nzo, nzf))
+                Assembler.do.process((nzo, nzf))
 
             else:
-                logging.warning(Ta('warn-unknownEncoding@1'), filename)
+                if file_has_articles(nzf):
+                    logging.warning(Ta('%s -> Unknown encoding'), filename)
 
         if post_done:
-            self.remove(nzo.nzo_id, add_to_history=False, cleanup=False)
-
             if not self.__nzo_list:
                 # Close server connections
-                if cfg.AUTODISCONNECT.get():
-                    sabnzbd.downloader.disconnect()
+                if cfg.autodisconnect():
+                    sabnzbd.downloader.Downloader.do.disconnect()
 
                 # Sets the end-of-queue back on if disabled
                 # adding an nzb and re-adding for more blocks disables it
@@ -612,7 +699,7 @@ class NzbQueue(TryList):
                     sabnzbd.QUEUECOMPLETEACTION_GO = True
 
             # Notify assembler to call postprocessor
-            sabnzbd.assembler.process((nzo, None))
+            Assembler.do.process((nzo, None))
 
 
     @synchronized(NZBQUEUE_LOCK)
@@ -622,7 +709,7 @@ class NzbQueue(TryList):
         pnfo_list = []
         for nzo in self.__nzo_list:
             pnfo = nzo.gather_info(for_cli = for_cli)
-            if nzo.get_status() != 'Paused':
+            if nzo.status != 'Paused':
                 bytes += pnfo[PNFO_BYTES_FIELD]
                 bytes_left += pnfo[PNFO_BYTES_LEFT_FIELD]
             pnfo_list.append(pnfo)
@@ -639,32 +726,20 @@ class NzbQueue(TryList):
         return empty
 
     @synchronized(NZBQUEUE_LOCK)
-    def cleanup_nzo(self, nzo):
-        nzo.purge_data()
-
-        sabnzbd.articlecache.method.purge_articles(nzo.saved_articles)
-
-        for hist_item in self.__downloaded_items:
-            # refresh fields & delete nzo reference
-            if hist_item.nzo and hist_item.nzo == nzo:
-                hist_item.cleanup()
-                logging.debug('%s cleaned up',
-                              nzo.get_dirname())
-
-    @synchronized(NZBQUEUE_LOCK)
-    def debug(self):
-        return (self.__downloaded_items[:], self.__nzo_list[:],
-                self.__nzo_table.copy(), self.try_list[:])
+    def cleanup_nzo(self, nzo, keep_basic=False, del_files=False):
+        nzo.purge_data(keep_basic, del_files)
 
+        ArticleCache.do.purge_articles(nzo.saved_articles)
 
     def get_urls(self):
         """ Return list of future-types needing URL """
         lst = []
         for nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
-            url = nzo.get_future()
-            if nzo.futuretype and url.lower().startswith('http'):
-                lst.append((url, nzo))
+            if nzo.futuretype:
+                url = nzo.url
+                if nzo.futuretype and url.lower().startswith('http'):
+                    lst.append((url, nzo))
         return lst
 
     def get_msgids(self):
@@ -672,7 +747,7 @@ class NzbQueue(TryList):
         lst = []
         for nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
-            msgid = nzo.get_future()
+            msgid = nzo.url
             if nzo.futuretype and (msgid.isdigit() or len(msgid)==5):
                 lst.append((msgid, nzo))
         return lst
@@ -683,8 +758,8 @@ class NzbQueue(TryList):
 #-------------------------------------------------------------------------------
 
 def _nzo_date_cmp(nzo1, nzo2):
-    avg_date1 = nzo1.get_avg_date()
-    avg_date2 = nzo2.get_avg_date()
+    avg_date1 = nzo1.avg_date
+    avg_date2 = nzo2.avg_date
 
     if avg_date1 is None and avg_date2 is None:
         return 0
@@ -697,16 +772,16 @@ def _nzo_date_cmp(nzo1, nzo2):
     return cmp(avg_date1, avg_date2)
 
 def _nzo_name_cmp(nzo1, nzo2):
-    return cmp(nzo1.get_filename(), nzo2.get_filename())
+    return cmp(nzo1.filename, nzo2.filename)
 
 def _nzo_size_cmp(nzo1, nzo2):
-    return cmp(nzo1.get_bytes(), nzo2.get_bytes())
+    return cmp(nzo1.bytes, nzo2.bytes)
 
 def sort_queue_function(nzo_list, method, reverse):
-    super_high_priority = [nzo for nzo in nzo_list if nzo.get_priority() == TOP_PRIORITY]
-    high_priority = [nzo for nzo in nzo_list if nzo.get_priority() == HIGH_PRIORITY]
-    normal_priority = [nzo for nzo in nzo_list if nzo.get_priority() == NORMAL_PRIORITY]
-    low_priority = [nzo for nzo in nzo_list if nzo.get_priority() == LOW_PRIORITY]
+    super_high_priority = [nzo for nzo in nzo_list if nzo.priority == TOP_PRIORITY]
+    high_priority = [nzo for nzo in nzo_list if nzo.priority == HIGH_PRIORITY]
+    normal_priority = [nzo for nzo in nzo_list if nzo.priority == NORMAL_PRIORITY]
+    low_priority = [nzo for nzo in nzo_list if nzo.priority == LOW_PRIORITY]
 
     super_high_priority.sort(cmp=method, reverse=reverse)
     high_priority.sort(cmp=method, reverse=reverse)
@@ -727,219 +802,37 @@ def sort_queue_function(nzo_list, method, reverse):
 
 
 
-#-------------------------------------------------------------------------------
-# NZBQ Wrappers
-
-__NZBQ = None  # Global pointer to NzbQueue instance
-
-def init():
-    global __NZBQ
-    if __NZBQ:
-        __NZBQ.__init__()
-    else:
-        __NZBQ = NzbQueue()
-
-def start():
-    global __NZBQ
-    if __NZBQ: __NZBQ.start()
-
-
-def stop():
-    global __NZBQ
-    if __NZBQ:
-        __NZBQ.stop()
-        try:
-            __NZBQ.join()
-        except:
-            pass
-
-def debug():
-    global __NZBQ
-    if __NZBQ: return __NZBQ.debug()
-
-def move_up_bulk(nzo_id, nzf_ids):
-    global __NZBQ
-    if __NZBQ: __NZBQ.move_up_bulk(nzo_id, nzf_ids)
-
-def move_top_bulk(nzo_id, nzf_ids):
-    global __NZBQ
-    if __NZBQ: __NZBQ.move_top_bulk(nzo_id, nzf_ids)
-
-def move_down_bulk(nzo_id, nzf_ids):
-    global __NZBQ
-    if __NZBQ: __NZBQ.move_down_bulk(nzo_id, nzf_ids)
-
-def move_bottom_bulk(nzo_id, nzf_ids):
-    global __NZBQ
-    if __NZBQ: __NZBQ.move_bottom_bulk(nzo_id, nzf_ids)
-
-def remove_nzo(nzo_id, add_to_history = True, unload=False):
-    global __NZBQ
-    if __NZBQ: __NZBQ.remove(nzo_id, add_to_history, unload)
-
-def remove_multiple_nzos(nzo_ids):
-    global __NZBQ
-    if __NZBQ: __NZBQ.remove_multiple(nzo_ids)
-
-def remove_all_nzo():
-    global __NZBQ
-    if __NZBQ: __NZBQ.remove_all()
-
-def remove_nzf(nzo_id, nzf_id):
-    global __NZBQ
-    if __NZBQ: __NZBQ.remove_nzf(nzo_id, nzf_id)
-
-def sort_by_avg_age(reverse=False):
-    global __NZBQ
-    if __NZBQ: __NZBQ.sort_by_avg_age(reverse)
-
-def sort_by_name(reverse=False):
-    global __NZBQ
-    if __NZBQ: __NZBQ.sort_by_name(reverse)
-
-def sort_by_size(reverse=False):
-    global __NZBQ
-    if __NZBQ: __NZBQ.sort_by_size(reverse)
-
-def change_opts(nzo_id, pp):
-    global __NZBQ
-    if __NZBQ: __NZBQ.change_opts(nzo_id, pp)
-
-def change_script(nzo_id, script):
-    global __NZBQ
-    if __NZBQ: __NZBQ.change_script(nzo_id, script)
-
-def change_cat(nzo_id, cat):
-    global __NZBQ
-    if __NZBQ: __NZBQ.change_cat(nzo_id, cat)
-
-def change_name(nzo_id, name):
-    global __NZBQ
-    if __NZBQ: __NZBQ.change_name(nzo_id, name)
-
-def get_article(host):
-    global __NZBQ
-    if __NZBQ: return __NZBQ.get_article(host)
-
-def has_articles():
-    global __NZBQ
-    if __NZBQ: return not __NZBQ.is_empty()
-
-def has_articles_for(server):
-    global __NZBQ
-    if __NZBQ: return __NZBQ.has_articles_for(server)
-
-def has_forced_items():
-    global __NZBQ
-    if __NZBQ: return __NZBQ.has_forced_items()
-
-def register_article(article):
-    global __NZBQ
-    if __NZBQ: return __NZBQ.register_article(article)
-
-def switch(nzo_id1, nzo_id2):
-    global __NZBQ
-    if __NZBQ:
-        return __NZBQ.switch(nzo_id1, nzo_id2)
-
-def get_position(nzo_id):
-    global __NZBQ
-    if __NZBQ:
-        return __NZBQ.get_position(nzo_id)
-
-def rename_nzo(nzo_id, name):
-    global __NZBQ
-    if __NZBQ: __NZBQ.change_name(nzo_id, name)
-
-def history_info():
-    global __NZBQ
-    if __NZBQ: return __NZBQ.history_info()
-
-def queue_info(for_cli = False):
-    global __NZBQ
-    if __NZBQ: return __NZBQ.queue_info(for_cli = for_cli)
-
-#def purge_history(job=None):
-#    global __NZBQ
-#    if __NZBQ: __NZBQ.purge(job)
-
-#def remove_multiple_history(jobs=None):
-#    global __NZBQ
-#    if __NZBQ: __NZBQ.remove_multiple_history(jobs)
-
-def get_msgids():
-    global __NZBQ
-    if __NZBQ: return __NZBQ.get_msgids()
-
-def get_urls():
-    global __NZBQ
-    if __NZBQ: return __NZBQ.get_urls()
-
-def pause_multiple_nzo(jobs):
-    global __NZBQ
-    if __NZBQ: __NZBQ.pause_multiple_nzo(jobs)
-
-def resume_multiple_nzo(jobs):
-    global __NZBQ
-    if __NZBQ: __NZBQ.resume_multiple_nzo(jobs)
-
-def cleanup_nzo(nzo):
-    global __NZBQ
-    if __NZBQ: __NZBQ.cleanup_nzo(nzo)
-
-def reset_try_lists(nzf = None, nzo = None):
-    global __NZBQ
-    if __NZBQ: __NZBQ.reset_try_lists(nzf, nzo)
-
-def reset_all_try_lists():
-    global __NZBQ
-    if __NZBQ: __NZBQ.reset_all_try_lists()
-
-def save():
-    global __NZBQ
-    if __NZBQ: __NZBQ.save()
-
-def generate_future(msg, pp, script, cat, url, priority, nzbname):
-    global __NZBQ
-    if __NZBQ: return __NZBQ.generate_future(msg, pp, script, cat, url, priority, nzbname)
-
-def set_top_only(value):
-    global __NZBQ
-    if __NZBQ: __NZBQ.set_top_only(value)
-
 #-------------------------------------------------------------------------------
 # Synchronized wrappers
 
 @synchronized_CV
 def add_nzo(nzo):
-    global __NZBQ
-    if __NZBQ: __NZBQ.add(nzo)
+    NzbQueue.do.add(nzo)
 
 @synchronized_CV
 def insert_future_nzo(future_nzo, filename, msgid, data, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None, nzo_info=None):
-    global __NZBQ
     if nzo_info is None:
         nzo_info = {}
-    if __NZBQ: __NZBQ.insert_future(future_nzo, filename, msgid, data, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname, nzo_info=nzo_info)
+    NzbQueue.do.insert_future(future_nzo, filename, msgid, data, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname, nzo_info=nzo_info)
 
 @synchronized_CV
-def set_priority(nzo_id, priority):
-    global __NZBQ
-    if __NZBQ:
-        return __NZBQ.set_priority(nzo_id, priority)
+def set_priority(nzo_ids, priority):
+    return NzbQueue.do.set_priority(nzo_ids, priority)
 
 @synchronized_CV
 def get_nzo(nzo_id):
-    global __NZBQ
-    if __NZBQ:
-        return __NZBQ.get_nzo(nzo_id)
-
-@synchronized_CV
-def set_priority_multiple(nzo_ids, priority):
-    global __NZBQ
-    if __NZBQ: return __NZBQ.set_priority_multiple(nzo_ids, priority)
+    return NzbQueue.do.get_nzo(nzo_id)
 
 @synchronized_CV
 def sort_queue(field, reverse=False):
-    global __NZBQ
-    if __NZBQ: __NZBQ.sort_queue(field, reverse)
+    NzbQueue.do.sort_queue(field, reverse)
+
+@synchronized_CV
+@synchronized(NZBQUEUE_LOCK)
+def repair_job(folder, new_nzb):
+    NzbQueue.do.repair_job(folder, new_nzb)
+
+@synchronized_CV
+@synchronized(NZBQUEUE_LOCK)
+def scan_jobs(all=False, action=True):
+    return NzbQueue.do.scan_jobs(all, action)
