@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2010 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,12 +27,11 @@ import logging
 import re
 
 import sabnzbd
-from sabnzbd.misc import move_to_path, cleanup_empty_directories, \
+from sabnzbd.misc import move_to_path, cleanup_empty_directories, get_unique_path, \
                          get_unique_filename, get_ext, renamer, remove_dir
 from sabnzbd.constants import series_match, date_match, year_match, sample_match
 import sabnzbd.cfg as cfg
-from sabnzbd.codecs import titler
-from sabnzbd.lang import Ta
+from sabnzbd.encoding import titler
 
 RE_SAMPLE = re.compile(sample_match, re.I)
 # Do not rename .vob files as they are usually DVD's
@@ -91,7 +90,7 @@ def move_to_parent_folder(workdir):
     return path1
 
 
-class Sorter:
+class Sorter(object):
     def __init__(self, cat):
         self.sorter = None
         self.type = None
@@ -131,28 +130,39 @@ class Sorter:
         if self.type == 'movie':
             move_to_parent = True
             # check if we should leave the files inside an extra folder
-            if cfg.MOVIE_EXTRA_FOLDER.get():
+            if cfg.movie_extra_folders():
                 #if there is a folder in the download, leave it in an extra folder
                 move_to_parent = not check_for_folder(workdir_complete)
             if move_to_parent:
                 workdir_complete = move_to_parent_folder(workdir_complete)
-            return workdir_complete
         else:
-            return move_to_parent_folder(workdir_complete)
+            workdir_complete = move_to_parent_folder(workdir_complete)
+        path, part = os.path.split(workdir_complete)
+        if '%fn' in part and self.sorter.fname:
+            old = workdir_complete
+            workdir_complete = os.path.join(path, part.replace('%fn', self.sorter.fname))
+            workdir_complete = get_unique_path(workdir_complete, create_dir=False)
+            try:
+                renamer(old, workdir_complete)
+            except:
+                logging.error(Ta('Cannot create directory %s'), workdir_complete)
+                workdir_complete = old
+        return workdir_complete
 
     def is_sortfile(self):
         return self.sort_file
 
-class SeriesSorter:
-    def __init__(self, dirname, path, cat):
+class SeriesSorter(object):
+    def __init__(self, dirname, path, cat, force=False):
         self.matched = False
 
         self.original_dirname = dirname
         self.original_path = path
         self.cat = cat
-        self.sort_string = cfg.TV_SORT_STRING.get()
-        self.cats = cfg.TV_CATEGORIES.get()
+        self.sort_string = cfg.tv_sort_string()
+        self.cats = cfg.tv_categories()
         self.filename_set = ''
+        self.fname = '' # Value for %fn substitution in folders
 
         self.match_obj = None
         self.extras = None
@@ -163,13 +173,13 @@ class SeriesSorter:
         self.show_info = {}
 
         #Check if it is a TV show on init()
-        self.match()
+        self.match(force)
 
 
-    def match(self):
+    def match(self, force=False):
         ''' Checks the regex for a match, if so set self.match to true '''
-        if cfg.ENABLE_TV_SORTING.get() and cfg.TV_SORT_STRING.get():
-            if (not self.cats) or (self.cat and self.cat.lower() in self.cats) or (not self.cat and 'None' in self.cats):
+        if force or (cfg.enable_tv_sorting() and cfg.tv_sort_string()):
+            if force or (not self.cats) or (self.cat and self.cat.lower() in self.cats) or (not self.cat and 'None' in self.cats):
                 #First check if the show matches TV episode regular expressions. Returns regex match object
                 self.match_obj, self.extras = check_regexs(self.original_dirname, series_match, double=True)
                 if self.match_obj:
@@ -275,7 +285,7 @@ class SeriesSorter:
             return True
 
         except:
-            logging.error(Ta('error-tvInfo@1'), self.original_dirname)
+            logging.error(Ta('Error getting TV info (%s)'), self.original_dirname)
             logging.info("Traceback: ", exc_info = True)
             return False
 
@@ -302,6 +312,9 @@ class SeriesSorter:
         mapping.append(('%s', self.show_info['season_num']))
         mapping.append(('%0s', self.show_info['season_num_alt']))
 
+        # Original dir name
+        mapping.append(('%dn', self.original_dirname))
+
         # Replace episode names
         if self.show_info['ep_name']:
             mapping.append(('%en', self.show_info['ep_name']))
@@ -375,6 +388,7 @@ class SeriesSorter:
         # >20MB
         if filepath and size > 20971520:
             tmp, ext = os.path.splitext(file)
+            self.fname = tmp
             newname = "%s%s" % (self.filename_set,ext)
             # Replace %fn with the original filename
             newname = newname.replace('%fn',tmp)
@@ -457,17 +471,18 @@ def check_for_sequence(regex, files):
 
 
 
-class GenericSorter:
+class GenericSorter(object):
     def __init__(self, dirname, path, cat):
         self.matched = False
 
         self.original_dirname = dirname
         self.original_path = path
-        self.sort_string = cfg.MOVIE_SORT_STRING.get()
-        self.extra = cfg.MOVIE_SORT_EXTRA.get()
-        self.cats = cfg.MOVIE_CATEGORIES.get()
+        self.sort_string = cfg.movie_sort_string()
+        self.extra = cfg.movie_sort_extra()
+        self.cats = cfg.movie_categories()
         self.cat = cat
         self.filename_set = ''
+        self.fname = '' # Value for %fn substitution in folders
 
         self.match_obj = None
 
@@ -481,7 +496,7 @@ class GenericSorter:
 
     def match(self):
         ''' Checks the category for a match, if so set self.match to true '''
-        if cfg.ENABLE_MOVIE_SORTING.get() and self.sort_string:
+        if cfg.enable_movie_sorting() and self.sort_string:
             #First check if the show matches TV episode regular expressions. Returns regex match object
             if (self.cat and self.cat.lower() in self.cats) or (not self.cat and 'None' in self.cats):
                 logging.debug("Movie Sorting - Starting folder sort (%s)", self.original_dirname)
@@ -609,6 +624,7 @@ class GenericSorter:
                 filepath = os.path.join(current_path, file)
             if os.path.exists(filepath):
                 tmp, ext = os.path.splitext(file)
+                self.fname = tmp
                 newname = "%s%s" % (self.filename_set,ext)
                 newname = newname.replace('%fn',tmp)
                 newpath = os.path.join(current_path, newname)
@@ -616,7 +632,7 @@ class GenericSorter:
                     logging.debug("Rename: %s to %s", filepath,newpath)
                     renamer(filepath,newpath)
                 except:
-                    logging.error(Ta('error-tvRename@2'), filepath, newpath)
+                    logging.error(Ta('Failed to rename: %s to %s'), filepath, newpath)
                     logging.info("Traceback: ", exc_info = True)
                 rename_similar(current_path, file, self.filename_set)
 
@@ -630,6 +646,7 @@ class GenericSorter:
                 for index, file in matched_files.iteritems():
                     filepath = os.path.join(current_path, file)
                     tmp, ext = os.path.splitext(file)
+                    self.fname = tmp
                     name = '%s%s' % (self.filename_set, self.extra)
                     name = name.replace('%1', str(index)).replace('%fn',tmp)
                     name = name + ext
@@ -638,23 +655,24 @@ class GenericSorter:
                         logging.debug("Rename: %s to %s", filepath,newpath)
                         renamer(filepath,newpath)
                     except:
-                        logging.error(Ta('error-tvRename@2'), filepath, newpath)
+                        logging.error(Ta('Failed to rename: %s to %s'), filepath, newpath)
                         logging.info("Traceback: ", exc_info = True)
                     rename_similar(current_path, file, self.filename_set)
             else:
                 logging.debug("Movie files not in sequence %s", _files)
 
 
-class DateSorter:
+class DateSorter(object):
     def __init__(self, dirname, path, cat):
         self.matched = False
 
         self.original_dirname = dirname
         self.original_path = path
-        self.sort_string = cfg.DATE_SORT_STRING.get()
-        self.cats = cfg.DATE_CATEGORIES.get()
+        self.sort_string = cfg.date_sort_string()
+        self.cats = cfg.date_categories()
         self.cat = cat
         self.filename_set = ''
+        self.fname = '' # Value for %fn substitution in folders
 
         self.match_obj = None
 
@@ -662,6 +680,7 @@ class DateSorter:
         self.date_type = None
 
         self.date_info = {}
+        self.final_path = ''
 
         # Check if we match the category in init()
         self.match()
@@ -669,7 +688,7 @@ class DateSorter:
 
     def match(self):
         ''' Checks the category for a match, if so set self.matched to true '''
-        if cfg.ENABLE_DATE_SORTING.get() and self.sort_string:
+        if cfg.enable_date_sorting() and self.sort_string:
             #First check if the show matches TV episode regular expressions. Returns regex match object
             if (self.cat and self.cat.lower() in self.cats) or (not self.cat and 'None' in self.cats):
                 self.match_obj, self.date_type = checkForDate(self.original_dirname, date_match)
@@ -808,6 +827,7 @@ class DateSorter:
                 if size > 130000000:
                     if 'sample' not in file:
                         tmp, ext = os.path.splitext(file)
+                        self.fname = tmp
                         newname = "%s%s" % (self.filename_set,ext)
                         newname = newname.replace('%fn',tmp)
                         newpath = os.path.join(current_path, newname)
@@ -816,7 +836,7 @@ class DateSorter:
                                 logging.debug("Rename: %s to %s", filepath,newpath)
                                 renamer(filepath,newpath)
                             except:
-                                logging.error(Ta('error-tvRename@2'), current_path, newpath)
+                                logging.error(Ta('Failed to rename: %s to %s'), current_path, newpath)
                                 logging.info("Traceback: ", exc_info = True)
                             rename_similar(current_path, file, self.filename_set)
                             break
@@ -856,7 +876,7 @@ def getTitles(match, name):
         name = name[:match.start()]
 
     # Replace .US. with (US)
-    if cfg.TV_SORT_COUNTRIES.get() == 1:
+    if cfg.tv_sort_countries() == 1:
         for rep in COUNTRY_REP:
             # (us) > (US)
             name = replace_word(name, rep.lower(), rep)
@@ -866,7 +886,7 @@ def getTitles(match, name):
             dotted_country = '.%s.' % (rep.strip('()'))
             name = replace_word(name, dotted_country, rep)
     # Remove .US. and (US)
-    elif cfg.TV_SORT_COUNTRIES.get() == 2:
+    elif cfg.tv_sort_countries() == 2:
         for rep in COUNTRY_REP:
             # Remove (US)
             name = replace_word(name, rep, '')
@@ -883,11 +903,11 @@ def getTitles(match, name):
     title = title.replace("'S", "'s")
 
     # Replace titled country names, (Us) with (US) and so on
-    if cfg.TV_SORT_COUNTRIES.get() == 1:
+    if cfg.tv_sort_countries() == 1:
         for rep in COUNTRY_REP:
             title = title.replace(titler(rep), rep)
     # Remove country names, ie (Us)
-    elif cfg.TV_SORT_COUNTRIES.get() == 2:
+    elif cfg.tv_sort_countries() == 2:
         for rep in COUNTRY_REP:
             title = title.replace(titler(rep), '').strip()
 
@@ -932,7 +952,9 @@ def getDescriptions(match, name):
     ep_name = ep_name.strip(' _.')
     if ep_name.startswith('-'):
         ep_name = ep_name.strip('- _.')
-    ep_name = ep_name.replace('.', ' ').replace('_', ' ')
+    if '.' in ep_name and ' ' not in ep_name:
+        ep_name = ep_name.replace('.', ' ')
+    ep_name = ep_name.replace('_', ' ')
     ep_name2 = ep_name.replace(" - ", "-").replace(" ", ".")
     ep_name3 = ep_name.replace(" ", "_")
     return ep_name, ep_name2, ep_name3
@@ -1005,7 +1027,7 @@ def rename_similar(path, file, name):
                         logging.debug("Rename: %s to %s", fpath,newpath)
                         renamer(fpath,newpath)
                     except:
-                        logging.error(Ta('error-tvSimRename@2'), path, newpath)
+                        logging.error(Ta('Failed to rename similar file: %s to %s'), path, newpath)
                         logging.info("Traceback: ", exc_info = True)
 
 
@@ -1017,11 +1039,12 @@ def check_regexs(filename, matchers, double=False):
     Returns the MatchObject if a match is made
     This version checks for an additional match
     """
-    '''if double:
-        matcher, extramatchers = matchers
-    else:
-        matcher = matchers
-        extramatchers = []'''
+    #if double:
+    #   matcher, extramatchers = matchers
+    #else:
+    #    matcher = matchers
+    #    extramatchers = []
+
     extras = []
     for expressions in matchers:
         expression, extramatchers = expressions
@@ -1067,3 +1090,32 @@ def is_full_path(file):
     except:
         pass
     return False
+
+
+def eval_sort(sorttype, expression, name=None):
+    """ Preview a sort expression, to be used by API """
+    from sabnzbd.api import Ttemplate
+    path = ''
+    if sorttype == 'series':
+        name = name or ('%s S01E03 - %s [DTS]' % (Ttemplate('show-name'), Ttemplate('ep-name')))
+        sorter = sabnzbd.tvsort.SeriesSorter(name, path, 'tv', force=True)
+    elif sorttype == 'generic':
+        name = name or (Ttemplate('movie-sp-name') + ' (2009)')
+        sorter = sabnzbd.tvsort.GenericSorter(name, path, 'tv')
+    elif sorttype == 'date':
+        name = name or (Ttemplate('show-name') + ' 2009-01-02')
+        sorter = sabnzbd.tvsort.DateSorter(name, path, 'tv')
+    else:
+        return None
+    sorter.sort_string = expression
+    sorter.matched = True
+    path = sorter.get_final_path()
+    path = os.path.normpath(os.path.join(path, sorter.filename_set))
+    if sorter.rename_or_not:
+        path += '.avi'
+    else:
+        if sabnzbd.WIN32:
+            path += '\\'
+        else:
+            path += '/'
+    return path
