@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2010 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@ import re
 import logging
 import Queue
 import urllib
-from threading import *
+from threading import Thread
 
 import socket
 try:
@@ -37,63 +37,35 @@ except:
 import sabnzbd
 import sabnzbd.misc as misc
 import sabnzbd.dirscanner as dirscanner
-import sabnzbd.nzbqueue as nzbqueue
+from sabnzbd.nzbqueue import NzbQueue
 import sabnzbd.cfg as cfg
-from sabnzbd.lang import Ta
-
-#------------------------------------------------------------------------------
-# Wrapper functions
-
-__GRABBER = None  # Global pointer to url-grabber instance
-
-def init():
-    global __GRABBER
-    if __GRABBER:
-        __GRABBER.__init__()
-    else:
-        __GRABBER = URLGrabber()
-
-def start():
-    global __GRABBER
-    if __GRABBER: __GRABBER.start()
-
-def add(url, future_nzo):
-    global __GRABBER
-    if __GRABBER: __GRABBER.add(url, future_nzo)
-
-def stop():
-    global __GRABBER
-    if __GRABBER:
-        __GRABBER.stop()
-        try:
-            __GRABBER.join()
-        except:
-            pass
-
-def alive():
-    global __GRABBER
-    if __GRABBER:
-        return __GRABBER.isAlive()
-    else:
-        return False
-
 
 #------------------------------------------------------------------------------
 _RETRIES = 10
 
 class URLGrabber(Thread):
+    do = None  # Link to instance of the thread
+
     def __init__(self):
         Thread.__init__(self)
         self.queue = Queue.Queue()
-        for tup in sabnzbd.nzbqueue.get_urls():
+        for tup in NzbQueue.do.get_urls():
             url, nzo = tup
             self.queue.put((url, nzo, _RETRIES))
         self.shutdown = False
+        self.matrix_wait = 0
+        URLGrabber.do = self
 
     def add(self, url, future_nzo):
         """ Add an URL to the URLGrabber queue """
         self.queue.put((url, future_nzo, _RETRIES))
 
+    def rm_bookmark(self, url):
+        """ Add removal request for nzbmatrix bookmark """
+        if 'nzbmatrix.com' in url and cfg.matrix_del_bookmark():
+            url = url.replace('download.php?', 'bookmarks.php?action=remove&')
+            self.queue.put((url, None, _RETRIES))
+
     def stop(self):
         logging.info('URLGrabber shutting down')
         self.shutdown = True
@@ -104,26 +76,40 @@ class URLGrabber(Thread):
         self.shutdown = False
 
         while not self.shutdown:
+            # Don't pound the website!
+            time.sleep(5.0)
+
             (url, future_nzo, retry_count) = self.queue.get()
             if not url:
                 continue
+            url = url.replace(' ', '')
 
             try:
-                # If nzo entry deleted, give up
-                try:
-                    deleted = future_nzo.deleted
-                except:
-                    deleted = True
-                if deleted:
-                    logging.debug('Dropping URL %s, job entry missing', url)
-                    continue
+                del_bookmark = not future_nzo
+                if future_nzo:
+                    # If nzo entry deleted, give up
+                    try:
+                        deleted = future_nzo.deleted
+                    except:
+                        deleted = True
+                    if deleted:
+                        logging.debug('Dropping URL %s, job entry missing', url)
+                        continue
 
                 # Add nzbmatrix credentials if needed
                 url, matrix_id = _matrix_url(url)
 
+                # When still waiting for nzbmatrix wait period, requeue
+                if matrix_id and self.matrix_wait > time.time():
+                    self.queue.put((url, future_nzo, retry_count))
+                    continue
+
                 # _grab_url cannot reside in a function, because the tempfile
                 # would not survive the end of the function
-                logging.info('Grabbing URL %s', url)
+                if del_bookmark:
+                    logging.info('Removing nzbmatrix bookmark %s', matrix_id)
+                else:
+                    logging.info('Grabbing URL %s', url)
                 opener = urllib.FancyURLopener({})
                 opener.prompt_user_passwd = None
                 opener.addheaders = []
@@ -131,6 +117,8 @@ class URLGrabber(Thread):
                 opener.addheader('Accept-encoding','gzip')
                 filename = None
                 category = None
+                length = 0
+                nzo_info = {}
                 try:
                     fn, header = opener.retrieve(url)
                 except:
@@ -146,11 +134,13 @@ class URLGrabber(Thread):
                         if item in ('category_id', 'x-dnzb-category'):
                             category = value
                         elif item in ('x-dnzb-moreinfo',):
-                            more_info = value
+                            nzo_info['more_info'] = value
                         elif item in ('x-dnzb-name',):
                             filename = value
                             if not filename.endswith('.nzb'):
                                 filename += '.nzb'
+                        elif item in ('content-length',):
+                            length = misc.int_conv(value)
 
                         if not filename:
                             for item in tup:
@@ -158,7 +148,12 @@ class URLGrabber(Thread):
                                     filename = item[item.index("filename=") + 9:].strip(';').strip('"')
 
                 if matrix_id:
-                    fn, msg, retry = _analyse_matrix(fn, matrix_id)
+                    fn, msg, retry, wait = _analyse_matrix(fn, matrix_id)
+                    if retry and wait > 0:
+                        self.matrix_wait = time.time() + wait
+                        logging.debug('Retry URL %s after waiting', url)
+                        self.queue.put((url, future_nzo, retry_count))
+                        continue
                     category = _MATRIX_MAP.get(category, category)
                 else:
                     msg = ''
@@ -170,38 +165,45 @@ class URLGrabber(Thread):
                     if retry_count > 0 and retry:
                         logging.info('Retry URL %s', url)
                         self.queue.put((url, future_nzo, retry_count))
-                    else:
+                    elif not del_bookmark:
                         misc.bad_fetch(future_nzo, url, msg, retry=True)
                     continue
 
+                if del_bookmark:
+                    continue
+
                 if not filename:
                     filename = os.path.basename(url) + '.nzb'
+                # Sanitize and trim name, preserving the extension
+                filename, ext = os.path.splitext(filename)
                 filename = misc.sanitize_foldername(filename)
-                _r, _u, _d = future_nzo.get_repair_opts()
-                pp = sabnzbd.opts_to_pp(_r, _u, _d)
-                script = future_nzo.get_script()
-                cat = future_nzo.get_cat()
-                if cat is None and category:
-                    cat = misc.CatConvert(category)
-                priority = future_nzo.get_priority()
-                nzbname = future_nzo.get_dirname_rename()
+                filename += '.' + misc.sanitize_foldername(ext)
+
+                pp = future_nzo.pp
+                script = future_nzo.script
+                cat = future_nzo.cat
+                if (cat is None or cat == '*') and category:
+                    cat = misc.cat_convert(category)
+                priority = future_nzo.priority
+                nzbname = future_nzo.custom_name
 
                 # Check if nzb file
                 if os.path.splitext(filename)[1].lower() in ('.nzb', '.gz'):
-                    res = dirscanner.ProcessSingleFile(filename, fn, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname)
+                    res = dirscanner.ProcessSingleFile(filename, fn, pp=pp, script=script, cat=cat, priority=priority, \
+                                                       nzbname=nzbname, nzo_info=nzo_info, url=future_nzo.url)
                     if res == 0:
-                        nzbqueue.remove_nzo(future_nzo.nzo_id, add_to_history=False, unload=True)
+                        NzbQueue.do.remove(future_nzo.nzo_id, add_to_history=False)
                     elif res == -2:
                         self.add(url, future_nzo)
-                    elif matrix_id:
-                        # Keep retrying NzbMatrix forever
+                    elif matrix_id and length > 0:
+                        # Keep retrying NzbMatrix forever (if file not empty)
                         self.add(url, future_nzo)
                     else:
                         misc.bad_fetch(future_nzo, url, retry=True, content=True)
                 # Check if a supported archive
                 else:
-                    if dirscanner.ProcessArchiveFile(filename, fn, pp, script, cat, priority=priority) == 0:
-                        nzbqueue.remove_nzo(future_nzo.nzo_id, add_to_history=False, unload=True)
+                    if dirscanner.ProcessArchiveFile(filename, fn, pp, script, cat, priority=priority, url=future_nzo.url) == 0:
+                        NzbQueue.do.remove(future_nzo.nzo_id, add_to_history=False)
                     else:
                         # Not a supported filetype, not an nzb (text/html ect)
                         try:
@@ -210,11 +212,9 @@ class URLGrabber(Thread):
                             pass
                         misc.bad_fetch(future_nzo, url, retry=True, content=True)
             except:
-               logging.error('URLGRABBER CRASHED', exc_info = True)
-               logging.info("URLGRABBER Traceback: ", exc_info = True)
+                logging.error('URLGRABBER CRASHED', exc_info=True)
+                logging.debug("URLGRABBER Traceback: ", exc_info=True)
 
-            # Don't pound the website!
-            time.sleep(5.0)
 
 
 
@@ -231,51 +231,57 @@ def _matrix_url(url):
     if m:
         matrix_id = m.group(2)
         if not _RE_NZBMATRIX_USER.search(url) or not _RE_NZBMATRIX_API.search(url):
-            user = urllib.quote_plus(cfg.MATRIX_USERNAME.get())
-            key = urllib.quote_plus(cfg.MATRIX_APIKEY.get())
+            user = urllib.quote_plus(cfg.matrix_username())
+            key = urllib.quote_plus(cfg.matrix_apikey())
             url = '%s://api.nzbmatrix.com/v1.1/download.php?id=%s&username=%s&apikey=%s' % \
                   (_PROTOCOL, matrix_id, user, key)
     return url, matrix_id
 
+
 _RE_MATRIX_ERR = re.compile(r'please_wait[_ ]+(\d+)', re.I)
 
 def _analyse_matrix(fn, matrix_id):
     """ Analyse respons of nzbmatrix
+        returns fn|None, error-message|None, retry, wait-seconds
     """
     msg = ''
     if not fn:
         # No response, just retry
-        return (None, msg, True)
+        return (None, msg, True, 0)
     try:
         f = open(fn, 'r')
         data = f.read(40)
         f.close()
     except:
-        return (None, msg, True)
+        return (None, msg, True, 0)
 
     # Check for an error response
     if data and data.startswith('error'):
-        # Check if we are required to wait - if so sleep the urlgrabber
-        m = _RE_MATRIX_ERR.search(data)
-        if m:
-            wait = int(m.group(1))
-            if wait:
-                logging.debug('Sleeping URL grabber %s sec', wait)
-                time.sleep(min(wait, 60))
-                # Return, but tell the urlgrabber to retry
-                return (None, msg, True)
+        wait = 0
+        # Check for daily limit
+        if 'daily_limit' in data or 'limit is reached' in data:
+            # Daily limit reached, just wait an hour before trying again
+            wait = 3600
         else:
-            # Clear error message, don't retry
-            msg = Ta('warn-matrixFail@1') % data
-            return (None, msg, False)
+            # Check if we are required to wait - if so sleep the urlgrabber
+            m = _RE_MATRIX_ERR.search(data)
+            if m:
+                wait = min(int(m.group(1)), 600)
+            else:
+                # Clear error message, don't retry
+                msg = Ta('Problem accessing nzbmatrix server (%s)') % data
+                return (None, msg, False, 0)
+        if wait:
+            logging.debug('Further NzbMatrix grabs after %s sec', wait)
+            # Return, but tell the urlgrabber to retry
+            return (None, msg, True, wait)
 
     if data.startswith("<!DOCTYPE"):
         # We got HTML, probably a temporary problem, keep trying
-        msg = Ta('warn-matrixBadRep@1') % matrix_id
-        return (None, msg, True)
-
-    return fn, msg, False
+        msg = Ta('Invalid nzbmatrix report number %s') % matrix_id
+        return (None, msg, True, 0)
 
+    return fn, msg, False, 0
 
 
 #------------------------------------------------------------------------------
