@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,22 +25,22 @@ import sys
 from sabnzbd.newswrapper import NewsWrapper
 from sabnzbd.downloader import Server, clues_login, clues_too_many
 from sabnzbd.config import get_servers
-from sabnzbd.codecs import xml_name
-from sabnzbd.interface import IntConv
-from sabnzbd.lang import T
+from sabnzbd.encoding import xml_name
+from sabnzbd.misc import int_conv
 
 def test_nntp_server_dict(kwargs):
     # Grab the host/port/user/pass/connections/ssl
     host = kwargs.get('host', '').strip()
     if not host:
-        return False, T('srv-noHost')
+        return False, T('The hostname is not set.')
     username = kwargs.get('username', '').strip()
     password = kwargs.get('password', '').strip()
-    connections = IntConv(kwargs.get('connections', 0))
+    server = kwargs.get('server', '').strip()
+    connections = int_conv(kwargs.get('connections', 0))
     if not connections:
-        return False, T('srv-noConnections')
-    ssl = IntConv(kwargs.get('ssl', 0))
-    port = IntConv(kwargs.get('port', 0))
+        return False, T('There are no connections set. Please set at least one connection.')
+    ssl = int_conv(kwargs.get('ssl', 0))
+    port = int_conv(kwargs.get('port', 0))
     if not port:
         if ssl:
             port = 563
@@ -48,45 +48,54 @@ def test_nntp_server_dict(kwargs):
             port = 119
 
 
-    return test_nntp_server(host, port, username=username, \
+    return test_nntp_server(host, port, server, username=username, \
                         password=password, ssl=ssl)
 
 
-def test_nntp_server(host, port, username=None, password=None, ssl=None):
+def test_nntp_server(host, port, server=None, username=None, password=None, ssl=None):
     ''' Will connect (blocking) to the nttp server and report back any errors '''
     timeout = 4.0
     if '*' in password and not password.strip('*'):
         # If the password is masked, try retrieving it from the config
-        servers = get_servers()
-        got_pass = False
-        for server in servers:
-            if host in server:
-                srv = servers[server]
-                password = srv.password.get()
+        if not server:
+            servers = get_servers()
+            got_pass = False
+            for server in servers:
+                if host in server:
+                    srv = servers[server]
+                    password = srv.password()
+                    got_pass = True
+        else:
+            srv = get_servers().get(server)
+            if srv:
+                password = srv.password()
                 got_pass = True
         if not got_pass:
-            return False, T('srv-starredPass')
+            return False, T('Password masked in ******, please re-enter')
     try:
-        s = Server(-1, host, port, timeout, 1, 0, ssl, username, password)
+        s = Server(-1, host, port, timeout, 0, 0, ssl, username, password)
     except:
-        return False, T('srv-invalidDetails')
+        return False, T('Invalid server details')
 
     try:
         nw = NewsWrapper(s, -1, block=True)
-        nw.init_connect()
+        nw.init_connect(None)
         while not nw.connected:
             nw.lines = []
             nw.recv_chunk(block=True)
-            nw.finish_connect()
+            nw.finish_connect(nw.lines[0][:3])
 
     except socket.timeout, e:
         if port != 119 and not ssl:
-            return False, T('srv-timedoutSSL')
+            return False, T('Timed out: Try enabling SSL or connecting on a different port.')
         else:
-            return False, T('srv-timedout')
+            return False, T('Timed out')
     except socket.error, e:
         return False, xml_name(str(e))
 
+    except TypeError, e:
+        return False, xml_name(T('Invalid server address.'))
+
     except:
         return False, xml_name(str(sys.exc_info()[1]))
 
@@ -103,19 +112,19 @@ def test_nntp_server(host, port, username=None, password=None, ssl=None):
     code = nw.lines[0][:3]
 
     if code == '480':
-        return False, T('srv-noAuth')
+        return False, T('Server requires username and password.')
 
     elif code == '100' or code.startswith('2') or code.startswith('4'):
-        return True, T('srv-success')
+        return True, T('Connection Successful!')
 
     elif code == '502' or clues_login(nw.lines[0]):
-        return False, T('srv-failedAuth')
+        return False, T('Authentication failed, check username/password.')
 
     elif clues_too_many(nw.lines[0]):
-        return False, T('srv-tooManyConnections')
+        return False, T('Too many connections, please pause downloading or try again later')
 
     else:
-        return False, T('srv-generalFail@1') % xml_name(nw.lines[0])
+        return False, T('Could not determine connection result (%s)') % xml_name(nw.lines[0])
 
     # Close the connection
     nw.terminate(quit=True)
